from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/tripod_mounts.scad'}", use_not_include=False)

class manfrotto_rc2_plate(OpenSCADObject):
    def __init__(self, chamfer=None, anchor=None, orient=None, spin=None, **kwargs):
       super().__init__("manfrotto_rc2_plate", {"chamfer" : chamfer, "anchor" : anchor, "orient" : orient, "spin" : spin, **kwargs})




#! /usr/bin/env python

# run extopenscad with -r 2 to get proper results:
#       extopenscad -r 2 examples/14-implicitCAD.escad

# this is the same example as 06-functions.py but with nice smooth implicitCAD
# roundings....

# this is how you activate the "implicit mode" of ExpSolid
# I couldn't figure out a nicer way to set a parameter which can be accessed
# during the import routine of exp_solid
#
# alternatively you can call the whole script with the --implicit parameter:
#   python3 examples/14-implicitCAD.py --implicit

import sys
sys.argv.append("--implicit")

from solid2 import *

def wheel():
    return cylinder(r=35, h=15, center=True).rotate(0, 90, 0)

def axle():
    a = cylinder(r=10, h=130, center=True).\
            rotate(0, 90, 0)

    w1 = wheel().left(67)
    w2 = wheel().right(67)

    return w1 + w2 + a

def torso():
    bottom = cube(100, 250, 50, center=True, r=10)
    top = cube(80, 100, 60, center=True, r=10)

    window_cube = cube(200, 55 ,50, center=True, r=10).down(10)
    top = difference(r=10) (
                top,
                (union(r=10) (window_cube, window_cube.rotate(0, 0, 90)))
            )

    return union(r=10)(bottom, top.up(50))

def car():
    t = torso()

    front_axle = axle().down(20).back(80)

    rear_axle = front_axle.forward(160)

    return union(r=3)(t, front_axle, rear_axle)

car().save_as_scad()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/transforms.scad'}", use_not_include=False)

_NO_ARG = OpenSCADConstant('_NO_ARG')
class move(OpenSCADObject):
    def __init__(self, v=None, p=None, **kwargs):
       super().__init__("move", {"v" : v, "p" : p, **kwargs})

class left(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("left", {"x" : x, "p" : p, **kwargs})

class right(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("right", {"x" : x, "p" : p, **kwargs})

class xmove(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("xmove", {"x" : x, "p" : p, **kwargs})

class fwd(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("fwd", {"y" : y, "p" : p, **kwargs})

class back(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("back", {"y" : y, "p" : p, **kwargs})

class ymove(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("ymove", {"y" : y, "p" : p, **kwargs})

class down(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("down", {"z" : z, "p" : p, **kwargs})

class up(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("up", {"z" : z, "p" : p, **kwargs})

class zmove(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("zmove", {"z" : z, "p" : p, **kwargs})

class rot(OpenSCADObject):
    def __init__(self, a=None, v=None, cp=None, _from=None, to=None, reverse=None, **kwargs):
       super().__init__("rot", {"a" : a, "v" : v, "cp" : cp, "_from" : _from, "to" : to, "reverse" : reverse, **kwargs})

class xrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("xrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class yrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("yrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class zrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("zrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class xscale(OpenSCADObject):
    def __init__(self, x=None, p=None, cp=None, **kwargs):
       super().__init__("xscale", {"x" : x, "p" : p, "cp" : cp, **kwargs})

class yscale(OpenSCADObject):
    def __init__(self, y=None, p=None, cp=None, **kwargs):
       super().__init__("yscale", {"y" : y, "p" : p, "cp" : cp, **kwargs})

class zscale(OpenSCADObject):
    def __init__(self, z=None, p=None, cp=None, **kwargs):
       super().__init__("zscale", {"z" : z, "p" : p, "cp" : cp, **kwargs})

class xflip(OpenSCADObject):
    def __init__(self, p=None, x=None, **kwargs):
       super().__init__("xflip", {"p" : p, "x" : x, **kwargs})

class yflip(OpenSCADObject):
    def __init__(self, p=None, y=None, **kwargs):
       super().__init__("yflip", {"p" : p, "y" : y, **kwargs})

class zflip(OpenSCADObject):
    def __init__(self, p=None, z=None, **kwargs):
       super().__init__("zflip", {"p" : p, "z" : z, **kwargs})

class frame_map(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, p=None, reverse=None, **kwargs):
       super().__init__("frame_map", {"x" : x, "y" : y, "z" : z, "p" : p, "reverse" : reverse, **kwargs})

class skew(OpenSCADObject):
    def __init__(self, p=None, sxy=None, sxz=None, syx=None, syz=None, szx=None, szy=None, **kwargs):
       super().__init__("skew", {"p" : p, "sxy" : sxy, "sxz" : sxz, "syx" : syx, "syz" : syz, "szx" : szx, "szy" : szy, **kwargs})

class move(OpenSCADObject):
    def __init__(self, v=None, p=None, **kwargs):
       super().__init__("move", {"v" : v, "p" : p, **kwargs})

class translate(OpenSCADObject):
    def __init__(self, v=None, p=None, **kwargs):
       super().__init__("translate", {"v" : v, "p" : p, **kwargs})

class left(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("left", {"x" : x, "p" : p, **kwargs})

class right(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("right", {"x" : x, "p" : p, **kwargs})

class xmove(OpenSCADObject):
    def __init__(self, x=None, p=None, **kwargs):
       super().__init__("xmove", {"x" : x, "p" : p, **kwargs})

class fwd(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("fwd", {"y" : y, "p" : p, **kwargs})

class back(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("back", {"y" : y, "p" : p, **kwargs})

class ymove(OpenSCADObject):
    def __init__(self, y=None, p=None, **kwargs):
       super().__init__("ymove", {"y" : y, "p" : p, **kwargs})

class down(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("down", {"z" : z, "p" : p, **kwargs})

class up(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("up", {"z" : z, "p" : p, **kwargs})

class zmove(OpenSCADObject):
    def __init__(self, z=None, p=None, **kwargs):
       super().__init__("zmove", {"z" : z, "p" : p, **kwargs})

class rot(OpenSCADObject):
    def __init__(self, a=None, v=None, cp=None, _from=None, to=None, reverse=None, p=None, _m=None, **kwargs):
       super().__init__("rot", {"a" : a, "v" : v, "cp" : cp, "_from" : _from, "to" : to, "reverse" : reverse, "p" : p, "_m" : _m, **kwargs})

class xrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("xrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class yrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("yrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class zrot(OpenSCADObject):
    def __init__(self, a=None, p=None, cp=None, **kwargs):
       super().__init__("zrot", {"a" : a, "p" : p, "cp" : cp, **kwargs})

class scale(OpenSCADObject):
    def __init__(self, v=None, p=None, cp=None, **kwargs):
       super().__init__("scale", {"v" : v, "p" : p, "cp" : cp, **kwargs})

class xscale(OpenSCADObject):
    def __init__(self, x=None, p=None, cp=None, **kwargs):
       super().__init__("xscale", {"x" : x, "p" : p, "cp" : cp, **kwargs})

class yscale(OpenSCADObject):
    def __init__(self, y=None, p=None, cp=None, **kwargs):
       super().__init__("yscale", {"y" : y, "p" : p, "cp" : cp, **kwargs})

class zscale(OpenSCADObject):
    def __init__(self, z=None, p=None, cp=None, **kwargs):
       super().__init__("zscale", {"z" : z, "p" : p, "cp" : cp, **kwargs})

class mirror(OpenSCADObject):
    def __init__(self, v=None, p=None, **kwargs):
       super().__init__("mirror", {"v" : v, "p" : p, **kwargs})

class xflip(OpenSCADObject):
    def __init__(self, p=None, x=None, **kwargs):
       super().__init__("xflip", {"p" : p, "x" : x, **kwargs})

class yflip(OpenSCADObject):
    def __init__(self, p=None, y=None, **kwargs):
       super().__init__("yflip", {"p" : p, "y" : y, **kwargs})

class zflip(OpenSCADObject):
    def __init__(self, p=None, z=None, **kwargs):
       super().__init__("zflip", {"p" : p, "z" : z, **kwargs})

class frame_map(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, p=None, reverse=None, **kwargs):
       super().__init__("frame_map", {"x" : x, "y" : y, "z" : z, "p" : p, "reverse" : reverse, **kwargs})

class skew(OpenSCADObject):
    def __init__(self, p=None, sxy=None, sxz=None, syx=None, syz=None, szx=None, szy=None, **kwargs):
       super().__init__("skew", {"p" : p, "sxy" : sxy, "sxz" : sxz, "syx" : syx, "syz" : syz, "szx" : szx, "szy" : szy, **kwargs})

class is_2d_transform(OpenSCADObject):
    def __init__(self, t=None, **kwargs):
       super().__init__("is_2d_transform", {"t" : t, **kwargs})

class apply(OpenSCADObject):
    def __init__(self, transform=None, points=None, **kwargs):
       super().__init__("apply", {"transform" : transform, "points" : points, **kwargs})

class _apply(OpenSCADObject):
    def __init__(self, transform=None, points=None, **kwargs):
       super().__init__("_apply", {"transform" : transform, "points" : points, **kwargs})





class OperatorBase:
    def union_op(self, x):
        """
        This makes u = a+b identical to:
        u = union()(a, b )
        """
        from . import builtins
        res = builtins.union()

        #add self or all its children to res
        if isinstance(self, builtins.union):
            for c in self.children:
                res.add(c)
        else:
            res.add(self)

        #add x or all its children to res
        if isinstance(x, builtins.union):
            for c in x.children:
                res.add(c)
        else:
            res.add(x)

        return res

    def difference_op(self, x):
        """
        This makes u = a - b identical to:
        u = difference()(a, b )
        """
        from . import builtins
        res = builtins.difference()

        if isinstance(self, builtins.difference) and len(self.children):
            for c in self.children:
                res.add(c)
        else:
            res.add(self)

        res.add(x)
        return res

    def intersection_op(self, x):
        """
        This makes u = a * b identical to:
        u = intersection()(a, b )
        """
        from . import builtins
        res = builtins.intersection()

        if isinstance(self, builtins.intersection) and len(self.children):
            for c in self.children:
                res.add(c)
        else:
            res.add(self)

        if isinstance(x, builtins.intersection):
            for c in x.children:
                res.add(c)
        else:
            res.add(x)

        return res

    def __add__(self, x): return self.union_op(x)
    def __or__(self, x): return self.union_op(x)
    def __radd__(self, x): return self.union_op(x)
    def __sub__(self, x): return self.difference_op(x)
    def __mul__(self, x): return self.intersection_op(x)
    def __and__(self, x): return self.intersection_op(x)
    def __invert__(self): from . import builtins; return builtins.debug()(self)




#! /usr/bin/env python

import sys
sys.argv.append("--implicit")

from solid2 import *

#these are the Implicit examples from https://www.implicitcad.org/examples

def ex1():
    return linear_extrude (height = 30, twist = 180, r=2) (
               union (r=2) (
                   square(x=[-2,2], y=[-10,10], r=2),
                   square(x=[-10,10], y=[-2,2], r=2),
               )
           )

def ex2():
    #function parameter are a little tricky, but they do work like this:
    twist_func = scad_inline_parameter_func("twist(h) = 90*cos(h*2*pi/40)")

    return linear_extrude (height = 40, twist = twist_func) (
               difference () (
                   shell(2) (circle (10)),
                   square(x=[0,20], y=[-4,4]),
               )
           ).rotate(0, 0, 90).left(50)

def ex3():
    return union() (
               cylinder(r=19, h=10, _fn=6, center=True),
               cylinder(r=10, h=40),
               rotate_extrude(4*360, translate=[0,38]) (
                   translate ([10,0])( square([8,4], center=True))
               ),
           ).right(50)

def ex4():
    height_func = scad_inline_parameter_func("height(x, y) = 10 + 5*cos(x) + 5*cos(y)")

    return linear_extrude(height = height_func) (circle(10)).left(100)

def ex5():
    twist_func = scad_inline_parameter_func("twist(h) = 35*cos(h*2*pi/60)")

    return linear_extrude (height = 40, twist = twist_func) (
               union ( r = 8) (
                   circle (10),
                   translate ([22,0]) (circle (10)),
                   translate ([0,22]) (circle (10)),
                   translate ([-22,0]) (circle (10)),
                   translate ([0,-22]) (circle (10)),
               )
           ).right(130)

(ex1() + ex2() + ex3() + ex4() + ex5()).save_as_scad()




from .scad_import import use
from .object_base import ObjectBase
from ..config import config

# ====================
# = dynamic builtins =
# ====================
"""
    This loads all the built in OpenSCAD functions (like circle, square, color,
    translate.....) from builtins.openscad file.
    I intentionally didn't use a *.scad file because it would be ignore through
    .gitignore und would be a pain to maintain (unless we remove *.scad from
    .gitignore, but that would cause a lot of generated files to show up while
    developing...).
"""
if config.use_implicit_builtins:
    use(config.builtins_file, skip_render=True)
else:
    from .builtin_primitives import *

# =============
# = modifiers =
# =============
class ModifierBase(ObjectBase):
    def __init__(self, child=None):
        super().__init__()
        if child:
            self.children += [child]

class debug(ModifierBase):
    def _render(self):
        return "#" + super()._render()

class background(ModifierBase):
    def _render(self):
        return "%" + super()._render()

class root(ModifierBase):
    def _render(self):
        return "!" + super()._render()

class disable(ModifierBase):
    def _render(self):
        return "*" + super()._render()

# ==============
# = Directions =
# ==============
def up(z): return translate((0, 0, z))
def down(z): return translate((0, 0, -z))
def right(x): return translate((x, 0, 0))
def left(x): return translate((-x, 0, 0))
def forward(y): return translate((0, y, 0))
def fwd(y): return forward(y)
def back(y): return translate((0, -y, 0))

def translateX(x) : return translate((x, 0, 0))
def translateY(y) : return translate((0, y, 0))
def translateZ(z) : return translate((0, 0, z))

def rotateX(x): return rotate((x, 0, 0))
def rotateY(y): return rotate((0, y, 0))
def rotateZ(z): return rotate((0, 0, z))

def scaleX(x): return scale((x, 0, 0))
def scaleY(y): return scale((0, y, 0))
def scaleZ(z): return scale((0, 0, z))

if not config.use_implicit_builtins:
    def resizeX(x): return resize((x, 0, 0))
    def resizeY(y): return resize((0, y, 0))
    def resizeZ(z): return resize((0, 0, z))

    def mirrorX(): return mirror((1, 0, 0))
    def mirrorY(): return mirror((0, 1, 0))
    def mirrorZ(): return mirror((0, 0, 1))

# ==============================================
# = overwrite translate, rotate, scale, mirror =
# ==============================================
"""
    This block overwrites translate, scale, mirror and rotate from builtins
    to allow "overloading" those functions.
    As result you don't need to pass vectors in as lists, you can provide
    them as single integers / floats.

    translate([1, 2, 3])  ->   translate(1, 2, 3)
    rotate([1, 2, 3])   -> rotate(1, 2, 3)
"""
def extract_size_list(size_count, *args):
    if len(args) <= 1:
        return args

    size_list = []
    args_copy = list(args)

    while args_copy and len(size_list) < size_count:
        a = args_copy.pop(0)
        if type(a) == int or type(a) == float:
            size_list += [a]
        else:
            args_copy = [a] + args_copy
            break

    if len(size_list) == size_count:
        return [size_list] + args_copy
    else:
        return size_list + args_copy

orig_translate = translate
def translate(*args, **kwargs):
    args = extract_size_list(3, *args)
    return orig_translate(*args, **kwargs)

orig_scale = scale
def scale(*args, **kwargs):
    args = extract_size_list(3, *args)
    return orig_scale(*args, **kwargs)

orig_rotate = rotate
def rotate(*args, **kwargs):
    args = extract_size_list(3, *args)
    return orig_rotate(*args, **kwargs)

orig_square = square
def square(*args, **kwargs):
    args = extract_size_list(2, *args)
    return orig_square(*args, **kwargs)

orig_cube = cube
def cube(*args, **kwargs):
    args = extract_size_list(3, *args)
    return orig_cube(*args, **kwargs)

if not config.use_implicit_builtins:
    orig_resize = resize
    def resize(*args, **kwargs):
        args = extract_size_list(3, *args)
        return orig_resize(*args, **kwargs)

    orig_mirror = mirror
    def mirror(*args, **kwargs):
        args = extract_size_list(3, *args)
        return orig_mirror(*args, **kwargs)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/sliders.scad'}", use_not_include=False)

class slider(OpenSCADObject):
    def __init__(self, l=None, w=None, h=None, base=None, wall=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("slider", {"l" : l, "w" : w, "h" : h, "base" : base, "wall" : wall, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rail(OpenSCADObject):
    def __init__(self, l=None, w=None, h=None, chamfer=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rail", {"l" : l, "w" : w, "h" : h, "chamfer" : chamfer, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class slider(OpenSCADObject):
    def __init__(self, l=None, w=None, h=None, base=None, wall=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("slider", {"l" : l, "w" : w, "h" : h, "base" : base, "wall" : wall, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rail(OpenSCADObject):
    def __init__(self, l=None, w=None, h=None, chamfer=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rail", {"l" : l, "w" : w, "h" : h, "chamfer" : chamfer, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})




#! /usr/bin/env python

from solid2 import *

c = cube([10, 20, 30])

s = debug()(sphere(10))

d = c - s

bg = background()(
        cylinder(r=4, h=30)
     )

(d + bg).save_as_scad()

#disable (*) and root (!) are accessible corresponding to debug and background
#NOTE: the __invert__ operator is mapped to debug:
#
#            debug()(cube(1))
#
#       can be expressed as:
#
#             ~cube(1)
#
#this file generates the following scad code:
#
#    union() {
#            difference() {
#                    cube(size = [10, 20, 30]);
#                    #sphere(r = 10);
#            };
#            %cylinder(h = 30, r = 4);
#    };




from ..config import config

def builtins():
    from . import builtins
    return builtins

class AccessSyntaxBase:
    if not config.use_implicit_builtins:
        def intersection_for(self, n):     return builtins().intersection_for(n)(self)
        def color(self, color, alpha=1.0): return builtins().color(color, alpha)(self)
        def hull(self):                    return builtins().hull()(self)
        def render(self, convexity=None):  return builtins().render(convexity)(self)
        def projection(self, cut=None):    return builtins().projection(cut=None)(self)

        def surface(self, file, center=None, convexity=None, invert=None):
            return builtins().surface(file, center, convexity, invert)(self)
        def offset(self, r=None, delta=None, chamfer=None, _fn=None):
            return builtins().offset(r, delta, chamfer, _fn)(self)

        def mirror(self, *args, **kwargs): return builtins().mirror(*args, **kwargs)(self)
        def resize(self, *args, **kwargs): return builtins().resize(*args, **kwargs)(self)

        def mirrorX(self, x): return builtins().mirrorX(x)(self)
        def mirrorY(self, y): return builtins().mirrorY(y)(self)
        def mirrorZ(self, z): return builtins().mirrorZ(z)(self)

        def resizeX(self, x): return builtins().resizeX(x)(self)
        def resizeY(self, y): return builtins().resizeY(y)(self)
        def resizeZ(self, z): return builtins().resizeZ(z)(self)

    def union(self):          return builtins().union()(self)
    def difference(self):     return builtins().difference()(self)
    def intersection(self):   return builtins().intersection()(self)


    def rotate_extrude(self, angle=None, convexity=None, _fn=None):
        return builtins().rotate_extrude(*args, **kwargs)(self)
    def linear_extrude(self, height=None, center=None, convexity=None, \
                       twist=None, slices=None, scale=None):
        return builtins().linear_extrude(height, center, convexity, twist, slices, scale)(self)

    def translate(self, *args, **kwargs): return builtins().translate(*args, **kwargs)(self)
    def scale(self, *args, **kwargs):     return builtins().scale(*args, **kwargs)(self)
    def rotate(self, *args, **kwargs):    return builtins().rotate(*args, **kwargs)(self)

    def down(self, z):    return builtins().down(z)(self)
    def up(self, z):      return builtins().up(z)(self)
    def left(self, x):    return builtins().left(x)(self)
    def right(self, x):   return builtins().right(x)(self)
    def back(self, y):    return builtins().back(y)(self)
    def fwd(self, y):     return builtins().fwd(y)(self)
    def forward(self, y): return builtins().fwd(y)(self)

    def translateX(self, x): return builtins().translateX(x)(self)
    def translateY(self, y): return builtins().translateY(y)(self)
    def translateZ(self, z): return builtins().translateZ(z)(self)

    def rotateX(self, x): return builtins().rotateX(x)(self)
    def rotateY(self, y): return builtins().rotateY(y)(self)
    def rotateZ(self, z): return builtins().rotateZ(z)(self)

    def scaleX(self, x): return builtins().scaleX(x)(self)
    def scaleY(self, y): return builtins().scaleY(y)(self)
    def scaleZ(self, z): return builtins().scaleZ(z)(self)

    def debug(self):      return builtins().debug()(self)
    def background(self): return builtins().background()(self)
    def root(self):       return builtins().root()(self)
    def disable(self):    return builtins().disable()(self)




from .object_base import OpenSCADObject
from .utils import escape_openscad_identifier

# =========================
# = creating the wrappers =
# =========================
def check_signature(name, args_def, kwargs_def, *args, **kwargs):
    #check whether the args and kwargs fit a function signature definition
    #defined with args_def and kwargs_def. args_def and kwargs_def are lists
    #of all parameter names the function {name} accepts

    filtered_kwargs = {k: kwargs[k] for k in kwargs if not k.startswith("_")}
    if len(args) + len(filtered_kwargs) > len(args_def) + len(kwargs_def):
        raise TypeError(f"too many arguments to {name}(...)")

    full_args_tuples = list(zip(args_def + kwargs_def, args))
    full_args_tuples += list(zip(filtered_kwargs.keys(), filtered_kwargs.values()))

    full_args_names = [x[0] for x in full_args_tuples]

    args_def_copy = args_def[:]
    kwargs_def_copy = kwargs_def[:]

    while full_args_names and (args_def_copy or kwargs_def_copy):
        a = full_args_names.pop()

        if a in args_def_copy:
            args_def_copy.remove(a)

        elif a in kwargs_def_copy:
            kwargs_def_copy.remove(a)

        else:
            raise TypeError(f"{name}(...) has no parameter {a} or it is " +\
                            f"already occupied by a positional argument")

    #are there still unmatched parameters left?
    if full_args_names:
        if not args_def_copy and not kwargs_def_copy:
            raise TypeError(f"{name}(...) too many arguments")
        else:
            assert(False)

    #are there still unmet args in args_def?
    if args_def_copy and not full_args_names:
        raise TypeError(f"not enough parameters to {name}(...)")

def create_openscad_wrapper_from_symbols(name, args, kwargs):

    #this is the function we'll bind to the init function of the new class
    #that we'll create to represent the openscad function
    def init_func(self, *args, **kwargs):
        def legacy_patch(kwargs):
            # this function patches the kwargs to be backward compatible
            import keyword
            if "segments" in kwargs.keys():
                kwargs["_fn"] = kwargs.pop("segments")
            keys_to_replace = []

            #replace keyword_ with _keyword
            for k in kwargs.keys():
                if k.endswith("_") and keyword.iskeyword(k[:-1]):
                    keys_to_replace.append(k)
            for k in keys_to_replace:
                kwargs["_" + k[:-1]] = kwargs.pop(k)

            #replace __[0-9]... with _[0-9]
            keys_to_replace = []
            for k in kwargs.keys():
                if k.startswith("__") and k[2].isdigit():
                    keys_to_replace.append(k)
            for k in keys_to_replace:
                kwargs["_" + k[2:]] = kwargs.pop(k)

        legacy_patch(kwargs)

        #check whether the *args and **kwargs meet our parameter definitions
        check_signature(name, args_def, kwargs_def, *args, **kwargs)

        #zip the args with the def dicts and update it with kwargs
        #to get a single complete kwargs list
        params = dict(zip(args_def + kwargs_def, args))
        params.update(kwargs)

        #call OpenSCADObject ctor
        return super(self.__class__, self).__init__(name, params)


    #escape all identifiers
    name = escape_openscad_identifier(name)
    args_def = list(map(escape_openscad_identifier, args))
    kwargs_def = list(map(escape_openscad_identifier, kwargs))

    #create the class and bind an "instance of" _init_func it's __init__ function
    class_declaration = type(name, (OpenSCADObject,), {"__init__" : init_func})

    #add the function signature as __doc__ string, so ExpSolidNamespace can
    #display it
    param_str = ",".join([str(x) for x in args])
    param_str += "," if param_str else ''
    param_str += ",".join([str(x) + "=..." for x in kwargs])
    class_declaration.__doc__ = f'{name}({param_str})'

    return class_declaration




from .scad_variable import *

fonts = []

def register_font(filename):
    fonts.append(filename)

def get_animation_time():
    return ScadValue("$t")

def set_global_fn(_fn):
    ScadVariable("$fn", _fn)

def set_global_fa(_fa):
    ScadVariable("$fa", _fa)

def set_global_fs(_fs):
    ScadVariable("$fs", _fs)

def set_global_viewport_translation(trans):
    ScadVariable("$vpt", trans)

def set_global_viewport_rotation(rot):
    ScadVariable("$vpr", rot)

def set_global_viewport_fov(fov):
    ScadVariable("$vpf", fov)

def set_global_viewport_distance(d):
    ScadVariable("$vpd", d)

def set_global_variable(var_name, value):
    ScadVariable(var_name, value)

def get_scad_header():
    base_str = "\n".join([f"use <{f}>" for f in fonts])
    base_str += "\n\n"
    base_str += "\n".join(ScadVariable.registered_variables.values())

    return base_str + "\n"




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/structs.scad'}", use_not_include=False)

class echo_struct(OpenSCADObject):
    def __init__(self, struct=None, name=None, **kwargs):
       super().__init__("echo_struct", {"struct" : struct, "name" : name, **kwargs})

class struct_set(OpenSCADObject):
    def __init__(self, struct=None, key=None, value=None, grow=None, **kwargs):
       super().__init__("struct_set", {"struct" : struct, "key" : key, "value" : value, "grow" : grow, **kwargs})

class _format_key(OpenSCADObject):
    def __init__(self, key=None, **kwargs):
       super().__init__("_format_key", {"key" : key, **kwargs})

class struct_remove(OpenSCADObject):
    def __init__(self, struct=None, key=None, **kwargs):
       super().__init__("struct_remove", {"struct" : struct, "key" : key, **kwargs})

class struct_val(OpenSCADObject):
    def __init__(self, struct=None, key=None, default=None, **kwargs):
       super().__init__("struct_val", {"struct" : struct, "key" : key, "default" : default, **kwargs})

class struct_keys(OpenSCADObject):
    def __init__(self, struct=None, **kwargs):
       super().__init__("struct_keys", {"struct" : struct, **kwargs})

class echo_struct(OpenSCADObject):
    def __init__(self, struct=None, name=None, **kwargs):
       super().__init__("echo_struct", {"struct" : struct, "name" : name, **kwargs})

class is_struct(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_struct", {"x" : x, **kwargs})




from pathlib import Path

from .scad_import import module_cache_by_resolved_filename, extra_scad_includes
from ..config import config

# =========================================
# = Rendering Python code to OpenSCAD code=
# =========================================
def render_to_stl_file(root, filename):
    scad_render_to_file(root, Path(filename).with_suffix(".stl.scad"))

    import subprocess
    args = ["openscad", "-o", filename, Path(filename).with_suffix(".stl.scad")]
    cp = subprocess.run(args, stdout=subprocess.PIPE)

    if cp.returncode == 0:
        return Path(filename).absolute().as_posix()

    return None

def scad_render(root, file_header = ''):
    #get a list of all used and included files
    includes = get_include_string()

    #call extensions pre_render
    from .extension_manager import default_extension_manager
    extensions_header_str = default_extension_manager.call_pre_render(root)
    extensions_header_str += "\n" if extensions_header_str else ''

    #wrap the extensions around the root node
    root = default_extension_manager.wrap_root_node(root)

    scad_body = root._render()

    #call extensions post_render
    extensions_footer_str = default_extension_manager.call_post_render(root)
    extensions_footer_str += "\n" if extensions_footer_str else ''

    return file_header + includes + extensions_header_str + scad_body \
                       + extensions_footer_str

def scad_render_to_file(scad_object, filename=None, out_dir='', file_header=''):

    if out_dir == None:
        out_dir = ''
    header = f"// Generated by ExpSolidPython\n" + "\n"

    rendered_string = scad_render(scad_object, file_header=file_header)

    return _write_to_file(rendered_string, filename, out_dir)

def _write_to_file(out_string, filename=None, outdir=''):
    outfile = filename

    if not outfile:
        suffix = ".scad" if not config.use_implicit_builtins else ".escad"

        #try to get the filename of the calling module
        import __main__
        if hasattr(__main__, "__file__"):
            #not called from a terminal
            calling_file = Path(__main__.__file__).absolute()
            outfile = calling_file.with_suffix(suffix)
        else:
            outfile = "expsolid_out" + suffix

    outpath = Path(outdir)
    if not outpath.exists():
        outpath.mkdir()

    outfile_path = outpath / Path(outfile)

    outfile_path.write_text(out_string)
    return outfile_path.absolute().as_posix()

def get_include_string():
    strings = []
    for file, use_not_include in extra_scad_includes:
        if use_not_include:
            strings.append(f"use <{file}>;")
        else:
            strings.append(f"include <{file}>;")

    for k, v in module_cache_by_resolved_filename.items():
        #skip builtins file
        if v[2]: #skip_render flag
            continue

        if v[1]:
            strings.append(f"use <{k}>;")
        else:
            strings.append(f"include <{k}>;")

    s = "\n".join(strings)
    s += "\n\n" if s else ''

    return s




#! /usr/bin/env python

from solid2 import *
from solid2.core.object_base import OpenSCADObject, ObjectBase

# ==============
# = Extensions =
# ==============
# expsolid is extendable through extensions. This (and the next) example show
# some usage of it.

# create a custom OpenSCADObject that maps to color(c="red")
class red(OpenSCADObject):
    def __init__(self):
        super().__init__(name="color", params={"c" : "red"})

# monkey patch it onto ObjectBase to allow "access-style" syntax
ObjectBase.red = lambda self: red()(self)

# a non sense object that's not an OpenSCADObject. You can use this to get
# "low-level" access if you don't want the typical OpenSCAD
# call(params)(children) syntax. For example the debug,background,.... modifiers
# are implemented like this (see core/builtins.py)
class non_sense_comment(ObjectBase):
    def _render(self):
        return "//non sense comment\n" + super()._render()

# A pre render extension. This hooks it into the "_render" routine. It will be
# called before the root node gets rendered. As a result you should(!) even be
# able to manipulate the whole tree (this is untested!), but at least to extract
# information from it, process it and use it to generate header contents
def non_sense_pre_render(root):

    def count_nense_recursive(node):
        count = 0
        if isinstance(node, non_sense_comment):
            count += 1

        for c in node.children:
            count += count_nense_recursive(c)

        return count

    count = count_nense_recursive(root)
    return f"//the root tree contains {count} non sense comment(s)\n"

# register the pre render extension.
from solid2.core.extension_manager import default_extension_manager
default_extension_manager.register_pre_render(non_sense_pre_render)
# ==============


cube1 = cube(10)
cube2 = cube(5).left(20)

commented_cube1 = non_sense_comment()(
                      cube1
                  )
# OpenSCAD-style syntax:
red_commented_cube1 = red()(
                          commented_cube1
                      )

commented_cube2 = non_sense_comment()(
                        cube2
                    )
# access-style syntax:
red_commented_cube2 = commented_cube2.\
                          red()

scene = red_commented_cube1 + red_commented_cube2
scene.save_as_scad()

# This generates the following output:
#
#     // Generated by ExpSolidPython
#
#     //the root tree contains 2 non sense comment(s)
#
#     union() {
#             color(c = "red") {
#                     //non sense comment
#                     cube(size = 10);
#             };
#             color(c = "red") {
#                     //non sense comment
#                     translate(v = [-20, 0, 0]) {
#                             cube(size = 5);
#                     };
#             };
#     };




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/version.scad'}", use_not_include=False)

BOSL_VERSION = OpenSCADConstant('BOSL_VERSION')
class bosl_required(OpenSCADObject):
    def __init__(self, version=None, **kwargs):
       super().__init__("bosl_required", {"version" : version, **kwargs})

class bosl_version(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("bosl_version", {**kwargs})

class bosl_version_num(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("bosl_version_num", {**kwargs})

class bosl_version_str(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("bosl_version_str", {**kwargs})

class _version_split_str(OpenSCADObject):
    def __init__(self, x=None, _i=None, _out=None, _num=None, **kwargs):
       super().__init__("_version_split_str", {"x" : x, "_i" : _i, "_out" : _out, "_num" : _num, **kwargs})

class version_to_list(OpenSCADObject):
    def __init__(self, version=None, **kwargs):
       super().__init__("version_to_list", {"version" : version, **kwargs})

class version_to_str(OpenSCADObject):
    def __init__(self, version=None, **kwargs):
       super().__init__("version_to_str", {"version" : version, **kwargs})

class version_to_num(OpenSCADObject):
    def __init__(self, version=None, **kwargs):
       super().__init__("version_to_num", {"version" : version, **kwargs})

class version_cmp(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("version_cmp", {"a" : a, "b" : b, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/modular_hose.scad'}", use_not_include=False)

_small_end = OpenSCADConstant('_small_end')
_big_end = OpenSCADConstant('_big_end')
_hose_waist = OpenSCADConstant('_hose_waist')
class modular_hose(OpenSCADObject):
    def __init__(self, size=None, type=None, clearance=None, waist_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("modular_hose", {"size" : size, "type" : type, "clearance" : clearance, "waist_len" : waist_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class modular_hose(OpenSCADObject):
    def __init__(self, size=None, type=None, clearance=None, waist_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("modular_hose", {"size" : size, "type" : type, "clearance" : clearance, "waist_len" : waist_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class modular_hose_radius(OpenSCADObject):
    def __init__(self, size=None, outer=None, **kwargs):
       super().__init__("modular_hose_radius", {"size" : size, "outer" : outer, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/beziers.scad'}", use_not_include=False)

_bezier_matrix_table = OpenSCADConstant('_bezier_matrix_table')
class debug_bezier(OpenSCADObject):
    def __init__(self, bezpath=None, width=None, N=None, **kwargs):
       super().__init__("debug_bezier", {"bezpath" : bezpath, "width" : width, "N" : N, **kwargs})

class debug_bezier_patches(OpenSCADObject):
    def __init__(self, patches=None, size=None, splinesteps=None, showcps=None, showdots=None, showpatch=None, convexity=None, style=None, **kwargs):
       super().__init__("debug_bezier_patches", {"patches" : patches, "size" : size, "splinesteps" : splinesteps, "showcps" : showcps, "showdots" : showdots, "showpatch" : showpatch, "convexity" : convexity, "style" : style, **kwargs})

class bezier_points(OpenSCADObject):
    def __init__(self, curve=None, u=None, **kwargs):
       super().__init__("bezier_points", {"curve" : curve, "u" : u, **kwargs})

class _signed_pascals_triangle(OpenSCADObject):
    def __init__(self, N=None, tri=None, **kwargs):
       super().__init__("_signed_pascals_triangle", {"N" : N, "tri" : tri, **kwargs})

class _compute_bezier_matrix(OpenSCADObject):
    def __init__(self, N=None, **kwargs):
       super().__init__("_compute_bezier_matrix", {"N" : N, **kwargs})

class _bezier_matrix(OpenSCADObject):
    def __init__(self, N=None, **kwargs):
       super().__init__("_bezier_matrix", {"N" : N, **kwargs})

class bezier_curve(OpenSCADObject):
    def __init__(self, bezier=None, splinesteps=None, endpoint=None, **kwargs):
       super().__init__("bezier_curve", {"bezier" : bezier, "splinesteps" : splinesteps, "endpoint" : endpoint, **kwargs})

class bezier_derivative(OpenSCADObject):
    def __init__(self, bezier=None, u=None, order=None, **kwargs):
       super().__init__("bezier_derivative", {"bezier" : bezier, "u" : u, "order" : order, **kwargs})

class bezier_tangent(OpenSCADObject):
    def __init__(self, bezier=None, u=None, **kwargs):
       super().__init__("bezier_tangent", {"bezier" : bezier, "u" : u, **kwargs})

class bezier_curvature(OpenSCADObject):
    def __init__(self, bezier=None, u=None, **kwargs):
       super().__init__("bezier_curvature", {"bezier" : bezier, "u" : u, **kwargs})

class bezier_closest_point(OpenSCADObject):
    def __init__(self, bezier=None, pt=None, max_err=None, u=None, end_u=None, **kwargs):
       super().__init__("bezier_closest_point", {"bezier" : bezier, "pt" : pt, "max_err" : max_err, "u" : u, "end_u" : end_u, **kwargs})

class bezier_length(OpenSCADObject):
    def __init__(self, bezier=None, start_u=None, end_u=None, max_deflect=None, **kwargs):
       super().__init__("bezier_length", {"bezier" : bezier, "start_u" : start_u, "end_u" : end_u, "max_deflect" : max_deflect, **kwargs})

class bezier_line_intersection(OpenSCADObject):
    def __init__(self, bezier=None, line=None, **kwargs):
       super().__init__("bezier_line_intersection", {"bezier" : bezier, "line" : line, **kwargs})

class bezpath_points(OpenSCADObject):
    def __init__(self, bezpath=None, curveind=None, u=None, N=None, **kwargs):
       super().__init__("bezpath_points", {"bezpath" : bezpath, "curveind" : curveind, "u" : u, "N" : N, **kwargs})

class bezpath_curve(OpenSCADObject):
    def __init__(self, bezpath=None, splinesteps=None, N=None, endpoint=None, **kwargs):
       super().__init__("bezpath_curve", {"bezpath" : bezpath, "splinesteps" : splinesteps, "N" : N, "endpoint" : endpoint, **kwargs})

class bezpath_closest_point(OpenSCADObject):
    def __init__(self, bezpath=None, pt=None, N=None, max_err=None, seg=None, min_seg=None, min_u=None, min_dist=None, **kwargs):
       super().__init__("bezpath_closest_point", {"bezpath" : bezpath, "pt" : pt, "N" : N, "max_err" : max_err, "seg" : seg, "min_seg" : min_seg, "min_u" : min_u, "min_dist" : min_dist, **kwargs})

class bezpath_length(OpenSCADObject):
    def __init__(self, bezpath=None, N=None, max_deflect=None, **kwargs):
       super().__init__("bezpath_length", {"bezpath" : bezpath, "N" : N, "max_deflect" : max_deflect, **kwargs})

class path_to_bezpath(OpenSCADObject):
    def __init__(self, path=None, closed=None, tangents=None, uniform=None, size=None, relsize=None, **kwargs):
       super().__init__("path_to_bezpath", {"path" : path, "closed" : closed, "tangents" : tangents, "uniform" : uniform, "size" : size, "relsize" : relsize, **kwargs})

class bezpath_close_to_axis(OpenSCADObject):
    def __init__(self, bezpath=None, axis=None, N=None, **kwargs):
       super().__init__("bezpath_close_to_axis", {"bezpath" : bezpath, "axis" : axis, "N" : N, **kwargs})

class bezpath_offset(OpenSCADObject):
    def __init__(self, offset=None, bezier=None, N=None, **kwargs):
       super().__init__("bezpath_offset", {"offset" : offset, "bezier" : bezier, "N" : N, **kwargs})

class bez_begin(OpenSCADObject):
    def __init__(self, pt=None, a=None, r=None, p=None, **kwargs):
       super().__init__("bez_begin", {"pt" : pt, "a" : a, "r" : r, "p" : p, **kwargs})

class bez_tang(OpenSCADObject):
    def __init__(self, pt=None, a=None, r1=None, r2=None, p=None, **kwargs):
       super().__init__("bez_tang", {"pt" : pt, "a" : a, "r1" : r1, "r2" : r2, "p" : p, **kwargs})

class bez_joint(OpenSCADObject):
    def __init__(self, pt=None, a1=None, a2=None, r1=None, r2=None, p1=None, p2=None, **kwargs):
       super().__init__("bez_joint", {"pt" : pt, "a1" : a1, "a2" : a2, "r1" : r1, "r2" : r2, "p1" : p1, "p2" : p2, **kwargs})

class bez_end(OpenSCADObject):
    def __init__(self, pt=None, a=None, r=None, p=None, **kwargs):
       super().__init__("bez_end", {"pt" : pt, "a" : a, "r" : r, "p" : p, **kwargs})

class is_bezier_patch(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_bezier_patch", {"x" : x, **kwargs})

class bezier_patch_flat(OpenSCADObject):
    def __init__(self, size=None, N=None, spin=None, orient=None, trans=None, **kwargs):
       super().__init__("bezier_patch_flat", {"size" : size, "N" : N, "spin" : spin, "orient" : orient, "trans" : trans, **kwargs})

class bezier_patch_reverse(OpenSCADObject):
    def __init__(self, patch=None, **kwargs):
       super().__init__("bezier_patch_reverse", {"patch" : patch, **kwargs})

class bezier_patch_points(OpenSCADObject):
    def __init__(self, patch=None, u=None, v=None, **kwargs):
       super().__init__("bezier_patch_points", {"patch" : patch, "u" : u, "v" : v, **kwargs})

class _bezier_rectangle(OpenSCADObject):
    def __init__(self, patch=None, splinesteps=None, style=None, **kwargs):
       super().__init__("_bezier_rectangle", {"patch" : patch, "splinesteps" : splinesteps, "style" : style, **kwargs})

class bezier_vnf(OpenSCADObject):
    def __init__(self, patches=None, splinesteps=None, style=None, **kwargs):
       super().__init__("bezier_vnf", {"patches" : patches, "splinesteps" : splinesteps, "style" : style, **kwargs})

class bezier_vnf_degenerate_patch(OpenSCADObject):
    def __init__(self, patch=None, splinesteps=None, reverse=None, return_edges=None, **kwargs):
       super().__init__("bezier_vnf_degenerate_patch", {"patch" : patch, "splinesteps" : splinesteps, "reverse" : reverse, "return_edges" : return_edges, **kwargs})

class bezier_patch_normals(OpenSCADObject):
    def __init__(self, patch=None, u=None, v=None, **kwargs):
       super().__init__("bezier_patch_normals", {"patch" : patch, "u" : u, "v" : v, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/paths.scad'}", use_not_include=False)

class is_path(OpenSCADObject):
    def __init__(self, list=None, dim=None, fast=None, **kwargs):
       super().__init__("is_path", {"list" : list, "dim" : dim, "fast" : fast, **kwargs})

class is_1region(OpenSCADObject):
    def __init__(self, path=None, name=None, **kwargs):
       super().__init__("is_1region", {"path" : path, "name" : name, **kwargs})

class force_path(OpenSCADObject):
    def __init__(self, path=None, name=None, **kwargs):
       super().__init__("force_path", {"path" : path, "name" : name, **kwargs})

class is_closed_path(OpenSCADObject):
    def __init__(self, path=None, eps=None, **kwargs):
       super().__init__("is_closed_path", {"path" : path, "eps" : eps, **kwargs})

class close_path(OpenSCADObject):
    def __init__(self, path=None, eps=None, **kwargs):
       super().__init__("close_path", {"path" : path, "eps" : eps, **kwargs})

class cleanup_path(OpenSCADObject):
    def __init__(self, path=None, eps=None, **kwargs):
       super().__init__("cleanup_path", {"path" : path, "eps" : eps, **kwargs})

class _path_select(OpenSCADObject):
    def __init__(self, path=None, s1=None, u1=None, s2=None, u2=None, closed=None, **kwargs):
       super().__init__("_path_select", {"path" : path, "s1" : s1, "u1" : u1, "s2" : s2, "u2" : u2, "closed" : closed, **kwargs})

class path_merge_collinear(OpenSCADObject):
    def __init__(self, path=None, closed=None, eps=None, **kwargs):
       super().__init__("path_merge_collinear", {"path" : path, "closed" : closed, "eps" : eps, **kwargs})

class path_length(OpenSCADObject):
    def __init__(self, path=None, closed=None, **kwargs):
       super().__init__("path_length", {"path" : path, "closed" : closed, **kwargs})

class path_segment_lengths(OpenSCADObject):
    def __init__(self, path=None, closed=None, **kwargs):
       super().__init__("path_segment_lengths", {"path" : path, "closed" : closed, **kwargs})

class path_length_fractions(OpenSCADObject):
    def __init__(self, path=None, closed=None, **kwargs):
       super().__init__("path_length_fractions", {"path" : path, "closed" : closed, **kwargs})

class _path_self_intersections(OpenSCADObject):
    def __init__(self, path=None, closed=None, eps=None, **kwargs):
       super().__init__("_path_self_intersections", {"path" : path, "closed" : closed, "eps" : eps, **kwargs})

class _sum_preserving_round(OpenSCADObject):
    def __init__(self, data=None, index=None, **kwargs):
       super().__init__("_sum_preserving_round", {"data" : data, "index" : index, **kwargs})

class subdivide_path(OpenSCADObject):
    def __init__(self, path=None, n=None, refine=None, maxlen=None, closed=None, exact=None, method=None, **kwargs):
       super().__init__("subdivide_path", {"path" : path, "n" : n, "refine" : refine, "maxlen" : maxlen, "closed" : closed, "exact" : exact, "method" : method, **kwargs})

class resample_path(OpenSCADObject):
    def __init__(self, path=None, n=None, spacing=None, closed=None, **kwargs):
       super().__init__("resample_path", {"path" : path, "n" : n, "spacing" : spacing, "closed" : closed, **kwargs})

class is_path_simple(OpenSCADObject):
    def __init__(self, path=None, closed=None, eps=None, **kwargs):
       super().__init__("is_path_simple", {"path" : path, "closed" : closed, "eps" : eps, **kwargs})

class path_closest_point(OpenSCADObject):
    def __init__(self, path=None, pt=None, closed=None, **kwargs):
       super().__init__("path_closest_point", {"path" : path, "pt" : pt, "closed" : closed, **kwargs})

class path_tangents(OpenSCADObject):
    def __init__(self, path=None, closed=None, uniform=None, **kwargs):
       super().__init__("path_tangents", {"path" : path, "closed" : closed, "uniform" : uniform, **kwargs})

class path_normals(OpenSCADObject):
    def __init__(self, path=None, tangents=None, closed=None, **kwargs):
       super().__init__("path_normals", {"path" : path, "tangents" : tangents, "closed" : closed, **kwargs})

class path_curvature(OpenSCADObject):
    def __init__(self, path=None, closed=None, **kwargs):
       super().__init__("path_curvature", {"path" : path, "closed" : closed, **kwargs})

class path_torsion(OpenSCADObject):
    def __init__(self, path=None, closed=None, **kwargs):
       super().__init__("path_torsion", {"path" : path, "closed" : closed, **kwargs})

class _path_cut_points(OpenSCADObject):
    def __init__(self, path=None, dists=None, closed=None, direction=None, **kwargs):
       super().__init__("_path_cut_points", {"path" : path, "dists" : dists, "closed" : closed, "direction" : direction, **kwargs})

class _path_cut_points_recurse(OpenSCADObject):
    def __init__(self, path=None, dists=None, closed=None, pind=None, dtotal=None, dind=None, result=None, **kwargs):
       super().__init__("_path_cut_points_recurse", {"path" : path, "dists" : dists, "closed" : closed, "pind" : pind, "dtotal" : dtotal, "dind" : dind, "result" : result, **kwargs})

class _path_cut_single(OpenSCADObject):
    def __init__(self, path=None, dist=None, closed=None, ind=None, eps=None, **kwargs):
       super().__init__("_path_cut_single", {"path" : path, "dist" : dist, "closed" : closed, "ind" : ind, "eps" : eps, **kwargs})

class _path_cuts_normals(OpenSCADObject):
    def __init__(self, path=None, cuts=None, dirs=None, closed=None, **kwargs):
       super().__init__("_path_cuts_normals", {"path" : path, "cuts" : cuts, "dirs" : dirs, "closed" : closed, **kwargs})

class _path_plane(OpenSCADObject):
    def __init__(self, path=None, ind=None, i=None, closed=None, **kwargs):
       super().__init__("_path_plane", {"path" : path, "ind" : ind, "i" : i, "closed" : closed, **kwargs})

class _path_cuts_dir(OpenSCADObject):
    def __init__(self, path=None, cuts=None, closed=None, eps=None, **kwargs):
       super().__init__("_path_cuts_dir", {"path" : path, "cuts" : cuts, "closed" : closed, "eps" : eps, **kwargs})

class path_cut(OpenSCADObject):
    def __init__(self, path=None, cutdist=None, closed=None, **kwargs):
       super().__init__("path_cut", {"path" : path, "cutdist" : cutdist, "closed" : closed, **kwargs})

class _path_cut_getpaths(OpenSCADObject):
    def __init__(self, path=None, cutlist=None, closed=None, **kwargs):
       super().__init__("_path_cut_getpaths", {"path" : path, "cutlist" : cutlist, "closed" : closed, **kwargs})

class _cut_to_seg_u_form(OpenSCADObject):
    def __init__(self, pathcut=None, path=None, closed=None, **kwargs):
       super().__init__("_cut_to_seg_u_form", {"pathcut" : pathcut, "path" : path, "closed" : closed, **kwargs})

class split_path_at_self_crossings(OpenSCADObject):
    def __init__(self, path=None, closed=None, eps=None, **kwargs):
       super().__init__("split_path_at_self_crossings", {"path" : path, "closed" : closed, "eps" : eps, **kwargs})

class _tag_self_crossing_subpaths(OpenSCADObject):
    def __init__(self, path=None, nonzero=None, closed=None, eps=None, **kwargs):
       super().__init__("_tag_self_crossing_subpaths", {"path" : path, "nonzero" : nonzero, "closed" : closed, "eps" : eps, **kwargs})

class polygon_parts(OpenSCADObject):
    def __init__(self, poly=None, nonzero=None, eps=None, **kwargs):
       super().__init__("polygon_parts", {"poly" : poly, "nonzero" : nonzero, "eps" : eps, **kwargs})

class _extreme_angle_fragment(OpenSCADObject):
    def __init__(self, seg=None, fragments=None, rightmost=None, eps=None, **kwargs):
       super().__init__("_extreme_angle_fragment", {"seg" : seg, "fragments" : fragments, "rightmost" : rightmost, "eps" : eps, **kwargs})

class _assemble_a_path_from_fragments(OpenSCADObject):
    def __init__(self, fragments=None, rightmost=None, startfrag=None, eps=None, **kwargs):
       super().__init__("_assemble_a_path_from_fragments", {"fragments" : fragments, "rightmost" : rightmost, "startfrag" : startfrag, "eps" : eps, **kwargs})

class _assemble_path_fragments(OpenSCADObject):
    def __init__(self, fragments=None, eps=None, _finished=None, **kwargs):
       super().__init__("_assemble_path_fragments", {"fragments" : fragments, "eps" : eps, "_finished" : _finished, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/rounding.scad'}", use_not_include=False)

class round_corners(OpenSCADObject):
    def __init__(self, path=None, method=None, radius=None, r=None, cut=None, joint=None, width=None, k=None, closed=None, verbose=None, **kwargs):
       super().__init__("round_corners", {"path" : path, "method" : method, "radius" : radius, "r" : r, "cut" : cut, "joint" : joint, "width" : width, "k" : k, "closed" : closed, "verbose" : verbose, **kwargs})

class smooth_path(OpenSCADObject):
    def __init__(self, path=None, tangents=None, size=None, relsize=None, splinesteps=None, uniform=None, closed=None, **kwargs):
       super().__init__("smooth_path", {"path" : path, "tangents" : tangents, "size" : size, "relsize" : relsize, "splinesteps" : splinesteps, "uniform" : uniform, "closed" : closed, **kwargs})

class path_join(OpenSCADObject):
    def __init__(self, paths=None, joint=None, k=None, relocate=None, closed=None, **kwargs):
       super().__init__("path_join", {"paths" : paths, "joint" : joint, "k" : k, "relocate" : relocate, "closed" : closed, **kwargs})

class offset_stroke(OpenSCADObject):
    def __init__(self, path=None, width=None, rounded=None, start=None, end=None, check_valid=None, quality=None, chamfer=None, closed=None, **kwargs):
       super().__init__("offset_stroke", {"path" : path, "width" : width, "rounded" : rounded, "start" : start, "end" : end, "check_valid" : check_valid, "quality" : quality, "chamfer" : chamfer, "closed" : closed, **kwargs})

class offset_sweep(OpenSCADObject):
    def __init__(self, path=None, height=None, bottom=None, top=None, h=None, l=None, offset=None, r=None, steps=None, quality=None, check_valid=None, extra=None, cut=None, chamfer_width=None, chamfer_height=None, joint=None, k=None, angle=None, convexity=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("offset_sweep", {"path" : path, "height" : height, "bottom" : bottom, "top" : top, "h" : h, "l" : l, "offset" : offset, "r" : r, "steps" : steps, "quality" : quality, "check_valid" : check_valid, "extra" : extra, "cut" : cut, "chamfer_width" : chamfer_width, "chamfer_height" : chamfer_height, "joint" : joint, "k" : k, "angle" : angle, "convexity" : convexity, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class convex_offset_extrude(OpenSCADObject):
    def __init__(self, height=None, bottom=None, top=None, h=None, l=None, length=None, offset=None, r=None, steps=None, extra=None, cut=None, chamfer_width=None, chamfer_height=None, joint=None, k=None, angle=None, convexity=None, thickness=None, **kwargs):
       super().__init__("convex_offset_extrude", {"height" : height, "bottom" : bottom, "top" : top, "h" : h, "l" : l, "length" : length, "offset" : offset, "r" : r, "steps" : steps, "extra" : extra, "cut" : cut, "chamfer_width" : chamfer_width, "chamfer_height" : chamfer_height, "joint" : joint, "k" : k, "angle" : angle, "convexity" : convexity, "thickness" : thickness, **kwargs})

class rounded_prism(OpenSCADObject):
    def __init__(self, bottom=None, top=None, joint_bot=None, joint_top=None, joint_sides=None, k_bot=None, k_top=None, k_sides=None, k=None, splinesteps=None, h=None, length=None, l=None, height=None, convexity=None, debug=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("rounded_prism", {"bottom" : bottom, "top" : top, "joint_bot" : joint_bot, "joint_top" : joint_top, "joint_sides" : joint_sides, "k_bot" : k_bot, "k_top" : k_top, "k_sides" : k_sides, "k" : k, "splinesteps" : splinesteps, "h" : h, "length" : length, "l" : l, "height" : height, "convexity" : convexity, "debug" : debug, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class bent_cutout_mask(OpenSCADObject):
    def __init__(self, r=None, thickness=None, path=None, radius=None, convexity=None, **kwargs):
       super().__init__("bent_cutout_mask", {"r" : r, "thickness" : thickness, "path" : path, "radius" : radius, "convexity" : convexity, **kwargs})

class join_prism(OpenSCADObject):
    def __init__(self, polygon=None, base=None, base_r=None, base_d=None, base_T=None, scale=None, prism_end_T=None, short=None, length=None, l=None, height=None, h=None, aux=None, aux_T=None, aux_r=None, aux_d=None, overlap=None, base_overlap=None, aux_overlap=None, n=None, base_n=None, end_n=None, aux_n=None, fillet=None, base_fillet=None, aux_fillet=None, end_round=None, k=None, base_k=None, aux_k=None, end_k=None, uniform=None, base_uniform=None, aux_uniform=None, debug=None, anchor=None, extent=None, cp=None, atype=None, orient=None, spin=None, convexity=None, **kwargs):
       super().__init__("join_prism", {"polygon" : polygon, "base" : base, "base_r" : base_r, "base_d" : base_d, "base_T" : base_T, "scale" : scale, "prism_end_T" : prism_end_T, "short" : short, "length" : length, "l" : l, "height" : height, "h" : h, "aux" : aux, "aux_T" : aux_T, "aux_r" : aux_r, "aux_d" : aux_d, "overlap" : overlap, "base_overlap" : base_overlap, "aux_overlap" : aux_overlap, "n" : n, "base_n" : base_n, "end_n" : end_n, "aux_n" : aux_n, "fillet" : fillet, "base_fillet" : base_fillet, "aux_fillet" : aux_fillet, "end_round" : end_round, "k" : k, "base_k" : base_k, "aux_k" : aux_k, "end_k" : end_k, "uniform" : uniform, "base_uniform" : base_uniform, "aux_uniform" : aux_uniform, "debug" : debug, "anchor" : anchor, "extent" : extent, "cp" : cp, "atype" : atype, "orient" : orient, "spin" : spin, "convexity" : convexity, **kwargs})

class round_corners(OpenSCADObject):
    def __init__(self, path=None, method=None, radius=None, r=None, cut=None, joint=None, width=None, k=None, closed=None, verbose=None, **kwargs):
       super().__init__("round_corners", {"path" : path, "method" : method, "radius" : radius, "r" : r, "cut" : cut, "joint" : joint, "width" : width, "k" : k, "closed" : closed, "verbose" : verbose, **kwargs})

class _smooth_bez_fill(OpenSCADObject):
    def __init__(self, points=None, k=None, **kwargs):
       super().__init__("_smooth_bez_fill", {"points" : points, "k" : k, **kwargs})

class _bezcorner(OpenSCADObject):
    def __init__(self, points=None, parm=None, **kwargs):
       super().__init__("_bezcorner", {"points" : points, "parm" : parm, **kwargs})

class _chamfcorner(OpenSCADObject):
    def __init__(self, points=None, parm=None, **kwargs):
       super().__init__("_chamfcorner", {"points" : points, "parm" : parm, **kwargs})

class _circlecorner(OpenSCADObject):
    def __init__(self, points=None, parm=None, **kwargs):
       super().__init__("_circlecorner", {"points" : points, "parm" : parm, **kwargs})

class _rounding_offsets(OpenSCADObject):
    def __init__(self, edgespec=None, z_dir=None, **kwargs):
       super().__init__("_rounding_offsets", {"edgespec" : edgespec, "z_dir" : z_dir, **kwargs})

class smooth_path(OpenSCADObject):
    def __init__(self, path=None, tangents=None, size=None, relsize=None, splinesteps=None, uniform=None, closed=None, **kwargs):
       super().__init__("smooth_path", {"path" : path, "tangents" : tangents, "size" : size, "relsize" : relsize, "splinesteps" : splinesteps, "uniform" : uniform, "closed" : closed, **kwargs})

class _scalar_to_vector(OpenSCADObject):
    def __init__(self, value=None, length=None, varname=None, **kwargs):
       super().__init__("_scalar_to_vector", {"value" : value, "length" : length, "varname" : varname, **kwargs})

class path_join(OpenSCADObject):
    def __init__(self, paths=None, joint=None, k=None, relocate=None, closed=None, **kwargs):
       super().__init__("path_join", {"paths" : paths, "joint" : joint, "k" : k, "relocate" : relocate, "closed" : closed, **kwargs})

class _path_join(OpenSCADObject):
    def __init__(self, paths=None, joint=None, k=None, i=None, result=None, relocate=None, closed=None, **kwargs):
       super().__init__("_path_join", {"paths" : paths, "joint" : joint, "k" : k, "i" : i, "result" : result, "relocate" : relocate, "closed" : closed, **kwargs})

class offset_stroke(OpenSCADObject):
    def __init__(self, path=None, width=None, rounded=None, start=None, end=None, check_valid=None, quality=None, chamfer=None, closed=None, **kwargs):
       super().__init__("offset_stroke", {"path" : path, "width" : width, "rounded" : rounded, "start" : start, "end" : end, "check_valid" : check_valid, "quality" : quality, "chamfer" : chamfer, "closed" : closed, **kwargs})

class os_pointed(OpenSCADObject):
    def __init__(self, dist=None, loc=None, **kwargs):
       super().__init__("os_pointed", {"dist" : dist, "loc" : loc, **kwargs})

class os_round(OpenSCADObject):
    def __init__(self, cut=None, angle=None, abs_angle=None, k=None, r=None, **kwargs):
       super().__init__("os_round", {"cut" : cut, "angle" : angle, "abs_angle" : abs_angle, "k" : k, "r" : r, **kwargs})

class os_flat(OpenSCADObject):
    def __init__(self, angle=None, abs_angle=None, **kwargs):
       super().__init__("os_flat", {"angle" : angle, "abs_angle" : abs_angle, **kwargs})

class angle_between_lines(OpenSCADObject):
    def __init__(self, line1=None, line2=None, **kwargs):
       super().__init__("angle_between_lines", {"line1" : line1, "line2" : line2, **kwargs})

class _parse_stroke_end(OpenSCADObject):
    def __init__(self, spec=None, name=None, **kwargs):
       super().__init__("_parse_stroke_end", {"spec" : spec, "name" : name, **kwargs})

class _stroke_end(OpenSCADObject):
    def __init__(self, width=None, left=None, right=None, spec=None, **kwargs):
       super().__init__("_stroke_end", {"width" : width, "left" : left, "right" : right, "spec" : spec, **kwargs})

class _path_line_intersection(OpenSCADObject):
    def __init__(self, path=None, line=None, ind=None, **kwargs):
       super().__init__("_path_line_intersection", {"path" : path, "line" : line, "ind" : ind, **kwargs})

class _make_offset_polyhedron(OpenSCADObject):
    def __init__(self, path=None, offsets=None, offset_type=None, flip_faces=None, quality=None, check_valid=None, offsetind=None, vertexcount=None, vertices=None, faces=None, **kwargs):
       super().__init__("_make_offset_polyhedron", {"path" : path, "offsets" : offsets, "offset_type" : offset_type, "flip_faces" : flip_faces, "quality" : quality, "check_valid" : check_valid, "offsetind" : offsetind, "vertexcount" : vertexcount, "vertices" : vertices, "faces" : faces, **kwargs})

class _struct_valid(OpenSCADObject):
    def __init__(self, spec=None, func=None, name=None, **kwargs):
       super().__init__("_struct_valid", {"spec" : spec, "func" : func, "name" : name, **kwargs})

class offset_sweep(OpenSCADObject):
    def __init__(self, path=None, height=None, bottom=None, top=None, h=None, l=None, length=None, offset=None, r=None, steps=None, quality=None, check_valid=None, extra=None, cut=None, chamfer_width=None, chamfer_height=None, joint=None, k=None, angle=None, **kwargs):
       super().__init__("offset_sweep", {"path" : path, "height" : height, "bottom" : bottom, "top" : top, "h" : h, "l" : l, "length" : length, "offset" : offset, "r" : r, "steps" : steps, "quality" : quality, "check_valid" : check_valid, "extra" : extra, "cut" : cut, "chamfer_width" : chamfer_width, "chamfer_height" : chamfer_height, "joint" : joint, "k" : k, "angle" : angle, **kwargs})

class os_circle(OpenSCADObject):
    def __init__(self, r=None, cut=None, extra=None, check_valid=None, quality=None, steps=None, offset=None, **kwargs):
       super().__init__("os_circle", {"r" : r, "cut" : cut, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "steps" : steps, "offset" : offset, **kwargs})

class os_teardrop(OpenSCADObject):
    def __init__(self, r=None, cut=None, extra=None, check_valid=None, quality=None, steps=None, offset=None, **kwargs):
       super().__init__("os_teardrop", {"r" : r, "cut" : cut, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "steps" : steps, "offset" : offset, **kwargs})

class os_chamfer(OpenSCADObject):
    def __init__(self, height=None, width=None, cut=None, angle=None, extra=None, check_valid=None, quality=None, steps=None, offset=None, **kwargs):
       super().__init__("os_chamfer", {"height" : height, "width" : width, "cut" : cut, "angle" : angle, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "steps" : steps, "offset" : offset, **kwargs})

class os_smooth(OpenSCADObject):
    def __init__(self, cut=None, joint=None, k=None, extra=None, check_valid=None, quality=None, steps=None, offset=None, **kwargs):
       super().__init__("os_smooth", {"cut" : cut, "joint" : joint, "k" : k, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "steps" : steps, "offset" : offset, **kwargs})

class os_profile(OpenSCADObject):
    def __init__(self, points=None, extra=None, check_valid=None, quality=None, offset=None, **kwargs):
       super().__init__("os_profile", {"points" : points, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "offset" : offset, **kwargs})

class os_mask(OpenSCADObject):
    def __init__(self, mask=None, out=None, extra=None, check_valid=None, quality=None, offset=None, **kwargs):
       super().__init__("os_mask", {"mask" : mask, "out" : out, "extra" : extra, "check_valid" : check_valid, "quality" : quality, "offset" : offset, **kwargs})

class convex_offset_extrude(OpenSCADObject):
    def __init__(self, height=None, bottom=None, top=None, h=None, l=None, length=None, offset=None, r=None, steps=None, extra=None, cut=None, chamfer_width=None, chamfer_height=None, joint=None, k=None, angle=None, convexity=None, thickness=None, **kwargs):
       super().__init__("convex_offset_extrude", {"height" : height, "bottom" : bottom, "top" : top, "h" : h, "l" : l, "length" : length, "offset" : offset, "r" : r, "steps" : steps, "extra" : extra, "cut" : cut, "chamfer_width" : chamfer_width, "chamfer_height" : chamfer_height, "joint" : joint, "k" : k, "angle" : angle, "convexity" : convexity, "thickness" : thickness, **kwargs})

class _remove_undefined_vals(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("_remove_undefined_vals", {"list" : list, **kwargs})

class _rp_compute_patches(OpenSCADObject):
    def __init__(self, top=None, bot=None, rtop=None, rsides=None, ktop=None, ksides=None, concave=None, **kwargs):
       super().__init__("_rp_compute_patches", {"top" : top, "bot" : bot, "rtop" : rtop, "rsides" : rsides, "ktop" : ktop, "ksides" : ksides, "concave" : concave, **kwargs})

class rounded_prism(OpenSCADObject):
    def __init__(self, bottom=None, top=None, joint_bot=None, joint_top=None, joint_sides=None, k_bot=None, k_top=None, k_sides=None, k=None, splinesteps=None, h=None, length=None, l=None, height=None, debug=None, **kwargs):
       super().__init__("rounded_prism", {"bottom" : bottom, "top" : top, "joint_bot" : joint_bot, "joint_top" : joint_top, "joint_sides" : joint_sides, "k_bot" : k_bot, "k_top" : k_top, "k_sides" : k_sides, "k" : k, "splinesteps" : splinesteps, "h" : h, "length" : length, "l" : l, "height" : height, "debug" : debug, **kwargs})

class _cyl_hole(OpenSCADObject):
    def __init__(self, r=None, path=None, **kwargs):
       super().__init__("_cyl_hole", {"r" : r, "path" : path, **kwargs})

class _circle_mask(OpenSCADObject):
    def __init__(self, r=None, **kwargs):
       super().__init__("_circle_mask", {"r" : r, **kwargs})

class bent_cutout_mask(OpenSCADObject):
    def __init__(self, r=None, thickness=None, path=None, radius=None, convexity=None, **kwargs):
       super().__init__("bent_cutout_mask", {"r" : r, "thickness" : thickness, "path" : path, "radius" : radius, "convexity" : convexity, **kwargs})

class join_prism(OpenSCADObject):
    def __init__(self, polygon=None, base=None, base_r=None, base_d=None, base_T=None, scale=None, prism_end_T=None, short=None, length=None, l=None, height=None, h=None, aux=None, aux_T=None, aux_r=None, aux_d=None, overlap=None, base_overlap=None, aux_overlap=None, n=None, base_n=None, aux_n=None, end_n=None, fillet=None, base_fillet=None, aux_fillet=None, end_round=None, k=None, base_k=None, aux_k=None, end_k=None, uniform=None, base_uniform=None, aux_uniform=None, debug=None, return_axis=None, **kwargs):
       super().__init__("join_prism", {"polygon" : polygon, "base" : base, "base_r" : base_r, "base_d" : base_d, "base_T" : base_T, "scale" : scale, "prism_end_T" : prism_end_T, "short" : short, "length" : length, "l" : l, "height" : height, "h" : h, "aux" : aux, "aux_T" : aux_T, "aux_r" : aux_r, "aux_d" : aux_d, "overlap" : overlap, "base_overlap" : base_overlap, "aux_overlap" : aux_overlap, "n" : n, "base_n" : base_n, "aux_n" : aux_n, "end_n" : end_n, "fillet" : fillet, "base_fillet" : base_fillet, "aux_fillet" : aux_fillet, "end_round" : end_round, "k" : k, "base_k" : base_k, "aux_k" : aux_k, "end_k" : end_k, "uniform" : uniform, "base_uniform" : base_uniform, "aux_uniform" : aux_uniform, "debug" : debug, "return_axis" : return_axis, **kwargs})

class _fix_angle_list(OpenSCADObject):
    def __init__(self, list=None, ind=None, result=None, **kwargs):
       super().__init__("_fix_angle_list", {"list" : list, "ind" : ind, "result" : result, **kwargs})

class _cyl_line_intersection(OpenSCADObject):
    def __init__(self, R=None, line=None, ref=None, **kwargs):
       super().__init__("_cyl_line_intersection", {"R" : R, "line" : line, "ref" : ref, **kwargs})

class _sphere_line_isect_best(OpenSCADObject):
    def __init__(self, R=None, line=None, ref=None, **kwargs):
       super().__init__("_sphere_line_isect_best", {"R" : R, "line" : line, "ref" : ref, **kwargs})

class _prism_line_isect(OpenSCADObject):
    def __init__(self, poly_pairs=None, line=None, ref=None, **kwargs):
       super().__init__("_prism_line_isect", {"poly_pairs" : poly_pairs, "line" : line, "ref" : ref, **kwargs})

class _prism_fillet(OpenSCADObject):
    def __init__(self, name=None, base=None, R=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, uniform=None, debug=None, **kwargs):
       super().__init__("_prism_fillet", {"name" : name, "base" : base, "R" : R, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "uniform" : uniform, "debug" : debug, **kwargs})

class _prism_fillet_plane(OpenSCADObject):
    def __init__(self, name=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, debug=None, **kwargs):
       super().__init__("_prism_fillet_plane", {"name" : name, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "debug" : debug, **kwargs})

class _prism_fillet_plane(OpenSCADObject):
    def __init__(self, name=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, debug=None, **kwargs):
       super().__init__("_prism_fillet_plane", {"name" : name, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "debug" : debug, **kwargs})

class _prism_fillet_cyl(OpenSCADObject):
    def __init__(self, name=None, R=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, uniform=None, debug=None, **kwargs):
       super().__init__("_prism_fillet_cyl", {"name" : name, "R" : R, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "uniform" : uniform, "debug" : debug, **kwargs})

class _prism_fillet_sphere(OpenSCADObject):
    def __init__(self, name=None, R=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, uniform=None, debug=None, **kwargs):
       super().__init__("_prism_fillet_sphere", {"name" : name, "R" : R, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "uniform" : uniform, "debug" : debug, **kwargs})

class _getnormal(OpenSCADObject):
    def __init__(self, polygon=None, index=None, u=None, **kwargs):
       super().__init__("_getnormal", {"polygon" : polygon, "index" : index, "u" : u, **kwargs})

class _polygon_step(OpenSCADObject):
    def __init__(self, poly=None, ind=None, u=None, dir=None, length=None, **kwargs):
       super().__init__("_polygon_step", {"poly" : poly, "ind" : ind, "u" : u, "dir" : dir, "length" : length, **kwargs})

class _prism_fillet_prism(OpenSCADObject):
    def __init__(self, name=None, basepoly=None, bot=None, top=None, d=None, k=None, N=None, overlap=None, uniform=None, debug=None, **kwargs):
       super().__init__("_prism_fillet_prism", {"name" : name, "basepoly" : basepoly, "bot" : bot, "top" : top, "d" : d, "k" : k, "N" : N, "overlap" : overlap, "uniform" : uniform, "debug" : debug, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/hingesnaps.scad'}", use_not_include=False)

class apply_folding_hinges_and_snaps(OpenSCADObject):
    def __init__(self, thick=None, foldangle=None, hinges=None, snaps=None, sockets=None, snaplen=None, snapdiam=None, hingegap=None, layerheight=None, **kwargs):
       super().__init__("apply_folding_hinges_and_snaps", {"thick" : thick, "foldangle" : foldangle, "hinges" : hinges, "snaps" : snaps, "sockets" : sockets, "snaplen" : snaplen, "snapdiam" : snapdiam, "hingegap" : hingegap, "layerheight" : layerheight, **kwargs})

class folding_hinge_mask(OpenSCADObject):
    def __init__(self, l=None, thick=None, layerheight=None, foldangle=None, hingegap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("folding_hinge_mask", {"l" : l, "thick" : thick, "layerheight" : layerheight, "foldangle" : foldangle, "hingegap" : hingegap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class snap_lock(OpenSCADObject):
    def __init__(self, thick=None, snaplen=None, snapdiam=None, layerheight=None, foldangle=None, hingegap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("snap_lock", {"thick" : thick, "snaplen" : snaplen, "snapdiam" : snapdiam, "layerheight" : layerheight, "foldangle" : foldangle, "hingegap" : hingegap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class snap_socket(OpenSCADObject):
    def __init__(self, thick=None, snaplen=None, snapdiam=None, layerheight=None, foldangle=None, hingegap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("snap_socket", {"thick" : thick, "snaplen" : snaplen, "snapdiam" : snapdiam, "layerheight" : layerheight, "foldangle" : foldangle, "hingegap" : hingegap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})




from ... import config

if config.use_implicit_builtins:
    raise Exception("ExpSolid: unfortunately ImplicitCAD can't handle bosl2...")

from .all import *

#TODO: extend this with all kinds of bosl2 features
from ...core.object_base import ObjectBase
ObjectBase.tag = lambda self, *args, **kwargs: tag(*args, **kwargs)(self)
ObjectBase.attach = lambda self, *args, **kwargs: attach(*args, **kwargs)(self)


#============ attachable add =============
#enhance the add function of the attachable OpenSCADObject so it can be used
#properly: cf. 07-libs-bosl2-attachable.py
attachable_default_add = attachable.add

def attachable_add(self, c):
    if len(self.children) == 0:
        attachable_default_add(self, c)
    elif len(self.children) == 1:
        attachable_default_add(self, union()(c))
    else:
        assert(len(self.children) == 2)
        self.children[1].add(c)

attachable.add = attachable_add
#============ attachable add end =============




#! /usr/bin/env python

from solid2 import *

# as soon as the objects get more complicated it makes sense to use functions
# to hierarchically assemble them.
# You can use python functions pretty much the same way as OpenSCAD modules
# except the children stuff. This is not possible with SolidPython but you
# should be albe to pass other OpenSCAD objects as parameters if neccessary.
# I haven't tried it but it should work....
#
# This simple example assembles a car similar to the one from the OpenSCAD
# tutorial:

def wheel():
    return cylinder(r=35, h=15, center=True).rotate(0, 90, 0)

def axle():
    a = cylinder(r=10, h=120, center=True).\
            rotate(0, 90, 0)

    w1 = wheel().left(70)
    w2 = wheel().right(70)

    return w1 + w2 + a

def torso():
    bottom = cube(100, 250, 50, center=True)
    top = cube(80, 100, 60, center=True)

    window_cube = cube(200, 55 ,50, center=True).down(10)
    top -= (window_cube + window_cube.rotate(0, 0, 90))

    return bottom + top.up(50)

def car():
    t = torso()

    front_axle = axle().down(20).back(80)

    rear_axle = front_axle.forward(160)

    return t + front_axle + rear_axle

car().save_as_scad()




#! /usr/bin/env python

from pathlib import Path
from openscad_extension_generator import generateStub

def generateBoslStd(bosl2_dir):
    stubFile = Path(__file__).absolute().parent / "bosl2" / "all.py"

    with open(stubFile, "w") as std_f:
        stdlibs = []
        with open(bosl2_dir / "std.scad") as f:
            for l in f.readlines():
                l = l.strip()
                if not l.startswith("include <"):
                    continue
                l = l.replace("include <", "").replace(">", "")
                std_f.write(f"from .{Path(l).stem} import *\n")
                stdlibs.append(l)

        for f in bosl2_dir.iterdir():
            if not f.suffix == ".scad" or f.name in stdlibs:
                continue
            if f.name in ["std.scad", "builtins.scad", "bosl1compat.scad"]:
                continue

            name = f.name.replace(".scad","")
            std_f.write(f"from . import {name}\n")


bosl2_dir = Path("../libs/BOSL2")

generateBoslStd(bosl2_dir)

for f in bosl2_dir.iterdir():
    if not f.suffix == ".scad":
        continue
    if f.name in ["std.scad", "builtins.scad", "bosl1compat.scad"]:
        continue

    generateStub(f, Path(__file__).parent / "bosl2", False)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/geometry.scad'}", use_not_include=False)

class hull_points(OpenSCADObject):
    def __init__(self, points=None, fast=None, **kwargs):
       super().__init__("hull_points", {"points" : points, "fast" : fast, **kwargs})

class is_point_on_line(OpenSCADObject):
    def __init__(self, point=None, line=None, bounded=None, eps=None, **kwargs):
       super().__init__("is_point_on_line", {"point" : point, "line" : line, "bounded" : bounded, "eps" : eps, **kwargs})

class _is_point_on_line(OpenSCADObject):
    def __init__(self, point=None, line=None, bounded=None, eps=None, **kwargs):
       super().__init__("_is_point_on_line", {"point" : point, "line" : line, "bounded" : bounded, "eps" : eps, **kwargs})

class _dist2line(OpenSCADObject):
    def __init__(self, d=None, n=None, **kwargs):
       super().__init__("_dist2line", {"d" : d, "n" : n, **kwargs})

class _valid_line(OpenSCADObject):
    def __init__(self, line=None, dim=None, eps=None, **kwargs):
       super().__init__("_valid_line", {"line" : line, "dim" : dim, "eps" : eps, **kwargs})

class _valid_plane(OpenSCADObject):
    def __init__(self, p=None, eps=None, **kwargs):
       super().__init__("_valid_plane", {"p" : p, "eps" : eps, **kwargs})

class _is_at_left(OpenSCADObject):
    def __init__(self, pt=None, line=None, eps=None, **kwargs):
       super().__init__("_is_at_left", {"pt" : pt, "line" : line, "eps" : eps, **kwargs})

class _degenerate_tri(OpenSCADObject):
    def __init__(self, tri=None, eps=None, **kwargs):
       super().__init__("_degenerate_tri", {"tri" : tri, "eps" : eps, **kwargs})

class _tri_class(OpenSCADObject):
    def __init__(self, tri=None, eps=None, **kwargs):
       super().__init__("_tri_class", {"tri" : tri, "eps" : eps, **kwargs})

class _pt_in_tri(OpenSCADObject):
    def __init__(self, point=None, tri=None, eps=None, **kwargs):
       super().__init__("_pt_in_tri", {"point" : point, "tri" : tri, "eps" : eps, **kwargs})

class _point_left_of_line2d(OpenSCADObject):
    def __init__(self, point=None, line=None, eps=None, **kwargs):
       super().__init__("_point_left_of_line2d", {"point" : point, "line" : line, "eps" : eps, **kwargs})

class is_collinear(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, eps=None, **kwargs):
       super().__init__("is_collinear", {"a" : a, "b" : b, "c" : c, "eps" : eps, **kwargs})

class point_line_distance(OpenSCADObject):
    def __init__(self, pt=None, line=None, bounded=None, **kwargs):
       super().__init__("point_line_distance", {"pt" : pt, "line" : line, "bounded" : bounded, **kwargs})

class segment_distance(OpenSCADObject):
    def __init__(self, seg1=None, seg2=None, eps=None, **kwargs):
       super().__init__("segment_distance", {"seg1" : seg1, "seg2" : seg2, "eps" : eps, **kwargs})

class line_normal(OpenSCADObject):
    def __init__(self, p1=None, p2=None, **kwargs):
       super().__init__("line_normal", {"p1" : p1, "p2" : p2, **kwargs})

class _general_line_intersection(OpenSCADObject):
    def __init__(self, s1=None, s2=None, eps=None, **kwargs):
       super().__init__("_general_line_intersection", {"s1" : s1, "s2" : s2, "eps" : eps, **kwargs})

class line_intersection(OpenSCADObject):
    def __init__(self, line1=None, line2=None, bounded1=None, bounded2=None, bounded=None, eps=None, **kwargs):
       super().__init__("line_intersection", {"line1" : line1, "line2" : line2, "bounded1" : bounded1, "bounded2" : bounded2, "bounded" : bounded, "eps" : eps, **kwargs})

class line_closest_point(OpenSCADObject):
    def __init__(self, line=None, pt=None, bounded=None, **kwargs):
       super().__init__("line_closest_point", {"line" : line, "pt" : pt, "bounded" : bounded, **kwargs})

class line_from_points(OpenSCADObject):
    def __init__(self, points=None, fast=None, eps=None, **kwargs):
       super().__init__("line_from_points", {"points" : points, "fast" : fast, "eps" : eps, **kwargs})

class is_coplanar(OpenSCADObject):
    def __init__(self, points=None, eps=None, **kwargs):
       super().__init__("is_coplanar", {"points" : points, "eps" : eps, **kwargs})

class plane3pt(OpenSCADObject):
    def __init__(self, p1=None, p2=None, p3=None, **kwargs):
       super().__init__("plane3pt", {"p1" : p1, "p2" : p2, "p3" : p3, **kwargs})

class plane3pt_indexed(OpenSCADObject):
    def __init__(self, points=None, i1=None, i2=None, i3=None, **kwargs):
       super().__init__("plane3pt_indexed", {"points" : points, "i1" : i1, "i2" : i2, "i3" : i3, **kwargs})

class plane_from_normal(OpenSCADObject):
    def __init__(self, normal=None, pt=None, **kwargs):
       super().__init__("plane_from_normal", {"normal" : normal, "pt" : pt, **kwargs})

class _eigenvals_symm_3(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("_eigenvals_symm_3", {"M" : M, **kwargs})

class _eigenvec_symm_3(OpenSCADObject):
    def __init__(self, M=None, evals=None, i=None, **kwargs):
       super().__init__("_eigenvec_symm_3", {"M" : M, "evals" : evals, "i" : i, **kwargs})

class _covariance_evec_eval(OpenSCADObject):
    def __init__(self, points=None, **kwargs):
       super().__init__("_covariance_evec_eval", {"points" : points, **kwargs})

class plane_from_points(OpenSCADObject):
    def __init__(self, points=None, fast=None, eps=None, **kwargs):
       super().__init__("plane_from_points", {"points" : points, "fast" : fast, "eps" : eps, **kwargs})

class plane_from_polygon(OpenSCADObject):
    def __init__(self, poly=None, fast=None, eps=None, **kwargs):
       super().__init__("plane_from_polygon", {"poly" : poly, "fast" : fast, "eps" : eps, **kwargs})

class plane_normal(OpenSCADObject):
    def __init__(self, plane=None, **kwargs):
       super().__init__("plane_normal", {"plane" : plane, **kwargs})

class plane_offset(OpenSCADObject):
    def __init__(self, plane=None, **kwargs):
       super().__init__("plane_offset", {"plane" : plane, **kwargs})

class _general_plane_line_intersection(OpenSCADObject):
    def __init__(self, plane=None, line=None, eps=None, **kwargs):
       super().__init__("_general_plane_line_intersection", {"plane" : plane, "line" : line, "eps" : eps, **kwargs})

class _normalize_plane(OpenSCADObject):
    def __init__(self, plane=None, **kwargs):
       super().__init__("_normalize_plane", {"plane" : plane, **kwargs})

class plane_line_intersection(OpenSCADObject):
    def __init__(self, plane=None, line=None, bounded=None, eps=None, **kwargs):
       super().__init__("plane_line_intersection", {"plane" : plane, "line" : line, "bounded" : bounded, "eps" : eps, **kwargs})

class plane_intersection(OpenSCADObject):
    def __init__(self, plane1=None, plane2=None, plane3=None, **kwargs):
       super().__init__("plane_intersection", {"plane1" : plane1, "plane2" : plane2, "plane3" : plane3, **kwargs})

class plane_line_angle(OpenSCADObject):
    def __init__(self, plane=None, line=None, **kwargs):
       super().__init__("plane_line_angle", {"plane" : plane, "line" : line, **kwargs})

class plane_closest_point(OpenSCADObject):
    def __init__(self, plane=None, points=None, **kwargs):
       super().__init__("plane_closest_point", {"plane" : plane, "points" : points, **kwargs})

class point_plane_distance(OpenSCADObject):
    def __init__(self, plane=None, point=None, **kwargs):
       super().__init__("point_plane_distance", {"plane" : plane, "point" : point, **kwargs})

class _pointlist_greatest_distance(OpenSCADObject):
    def __init__(self, points=None, plane=None, **kwargs):
       super().__init__("_pointlist_greatest_distance", {"points" : points, "plane" : plane, **kwargs})

class are_points_on_plane(OpenSCADObject):
    def __init__(self, points=None, plane=None, eps=None, **kwargs):
       super().__init__("are_points_on_plane", {"points" : points, "plane" : plane, "eps" : eps, **kwargs})

class _is_point_above_plane(OpenSCADObject):
    def __init__(self, plane=None, point=None, **kwargs):
       super().__init__("_is_point_above_plane", {"plane" : plane, "point" : point, **kwargs})

class circle_line_intersection(OpenSCADObject):
    def __init__(self, r=None, cp=None, line=None, bounded=None, d=None, eps=None, **kwargs):
       super().__init__("circle_line_intersection", {"r" : r, "cp" : cp, "line" : line, "bounded" : bounded, "d" : d, "eps" : eps, **kwargs})

class _circle_or_sphere_line_intersection(OpenSCADObject):
    def __init__(self, r=None, cp=None, line=None, bounded=None, d=None, eps=None, **kwargs):
       super().__init__("_circle_or_sphere_line_intersection", {"r" : r, "cp" : cp, "line" : line, "bounded" : bounded, "d" : d, "eps" : eps, **kwargs})

class circle_circle_intersection(OpenSCADObject):
    def __init__(self, r1=None, cp1=None, r2=None, cp2=None, eps=None, d1=None, d2=None, **kwargs):
       super().__init__("circle_circle_intersection", {"r1" : r1, "cp1" : cp1, "r2" : r2, "cp2" : cp2, "eps" : eps, "d1" : d1, "d2" : d2, **kwargs})

class circle_2tangents(OpenSCADObject):
    def __init__(self, r=None, pt1=None, pt2=None, pt3=None, tangents=None, d=None, **kwargs):
       super().__init__("circle_2tangents", {"r" : r, "pt1" : pt1, "pt2" : pt2, "pt3" : pt3, "tangents" : tangents, "d" : d, **kwargs})

class circle_3points(OpenSCADObject):
    def __init__(self, pt1=None, pt2=None, pt3=None, **kwargs):
       super().__init__("circle_3points", {"pt1" : pt1, "pt2" : pt2, "pt3" : pt3, **kwargs})

class circle_point_tangents(OpenSCADObject):
    def __init__(self, r=None, cp=None, pt=None, d=None, **kwargs):
       super().__init__("circle_point_tangents", {"r" : r, "cp" : cp, "pt" : pt, "d" : d, **kwargs})

class circle_circle_tangents(OpenSCADObject):
    def __init__(self, r1=None, cp1=None, r2=None, cp2=None, d1=None, d2=None, **kwargs):
       super().__init__("circle_circle_tangents", {"r1" : r1, "cp1" : cp1, "r2" : r2, "cp2" : cp2, "d1" : d1, "d2" : d2, **kwargs})

class _noncollinear_triple(OpenSCADObject):
    def __init__(self, points=None, error=None, eps=None, **kwargs):
       super().__init__("_noncollinear_triple", {"points" : points, "error" : error, "eps" : eps, **kwargs})

class sphere_line_intersection(OpenSCADObject):
    def __init__(self, r=None, cp=None, line=None, bounded=None, d=None, eps=None, **kwargs):
       super().__init__("sphere_line_intersection", {"r" : r, "cp" : cp, "line" : line, "bounded" : bounded, "d" : d, "eps" : eps, **kwargs})

class polygon_area(OpenSCADObject):
    def __init__(self, poly=None, signed=None, **kwargs):
       super().__init__("polygon_area", {"poly" : poly, "signed" : signed, **kwargs})

class centroid(OpenSCADObject):
    def __init__(self, object=None, eps=None, **kwargs):
       super().__init__("centroid", {"object" : object, "eps" : eps, **kwargs})

class _region_centroid(OpenSCADObject):
    def __init__(self, region=None, eps=None, **kwargs):
       super().__init__("_region_centroid", {"region" : region, "eps" : eps, **kwargs})

class _polygon_centroid(OpenSCADObject):
    def __init__(self, poly=None, eps=None, **kwargs):
       super().__init__("_polygon_centroid", {"poly" : poly, "eps" : eps, **kwargs})

class polygon_normal(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("polygon_normal", {"poly" : poly, **kwargs})

class _point_above_below_segment(OpenSCADObject):
    def __init__(self, point=None, edge=None, **kwargs):
       super().__init__("_point_above_below_segment", {"point" : point, "edge" : edge, **kwargs})

class point_in_polygon(OpenSCADObject):
    def __init__(self, point=None, poly=None, nonzero=None, eps=None, **kwargs):
       super().__init__("point_in_polygon", {"point" : point, "poly" : poly, "nonzero" : nonzero, "eps" : eps, **kwargs})

class polygon_line_intersection(OpenSCADObject):
    def __init__(self, poly=None, line=None, bounded=None, nonzero=None, eps=None, **kwargs):
       super().__init__("polygon_line_intersection", {"poly" : poly, "line" : line, "bounded" : bounded, "nonzero" : nonzero, "eps" : eps, **kwargs})

class _merge_segments(OpenSCADObject):
    def __init__(self, insegs=None, outsegs=None, eps=None, i=None, **kwargs):
       super().__init__("_merge_segments", {"insegs" : insegs, "outsegs" : outsegs, "eps" : eps, "i" : i, **kwargs})

class polygon_triangulate(OpenSCADObject):
    def __init__(self, poly=None, ind=None, error=None, eps=None, **kwargs):
       super().__init__("polygon_triangulate", {"poly" : poly, "ind" : ind, "error" : error, "eps" : eps, **kwargs})

class _triangulate(OpenSCADObject):
    def __init__(self, poly=None, ind=None, error=None, eps=None, tris=None, **kwargs):
       super().__init__("_triangulate", {"poly" : poly, "ind" : ind, "error" : error, "eps" : eps, "tris" : tris, **kwargs})

class _get_ear(OpenSCADObject):
    def __init__(self, poly=None, ind=None, eps=None, _i=None, **kwargs):
       super().__init__("_get_ear", {"poly" : poly, "ind" : ind, "eps" : eps, "_i" : _i, **kwargs})

class _none_inside(OpenSCADObject):
    def __init__(self, idxs=None, poly=None, p0=None, p1=None, p2=None, eps=None, i=None, **kwargs):
       super().__init__("_none_inside", {"idxs" : idxs, "poly" : poly, "p0" : p0, "p1" : p1, "p2" : p2, "eps" : eps, "i" : i, **kwargs})

class is_polygon_clockwise(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("is_polygon_clockwise", {"poly" : poly, **kwargs})

class clockwise_polygon(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("clockwise_polygon", {"poly" : poly, **kwargs})

class ccw_polygon(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("ccw_polygon", {"poly" : poly, **kwargs})

class reverse_polygon(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("reverse_polygon", {"poly" : poly, **kwargs})

class reindex_polygon(OpenSCADObject):
    def __init__(self, reference=None, poly=None, return_error=None, **kwargs):
       super().__init__("reindex_polygon", {"reference" : reference, "poly" : poly, "return_error" : return_error, **kwargs})

class align_polygon(OpenSCADObject):
    def __init__(self, reference=None, poly=None, angles=None, cp=None, trans=None, return_ind=None, **kwargs):
       super().__init__("align_polygon", {"reference" : reference, "poly" : poly, "angles" : angles, "cp" : cp, "trans" : trans, "return_ind" : return_ind, **kwargs})

class are_polygons_equal(OpenSCADObject):
    def __init__(self, poly1=None, poly2=None, eps=None, **kwargs):
       super().__init__("are_polygons_equal", {"poly1" : poly1, "poly2" : poly2, "eps" : eps, **kwargs})

class _are_polygons_equal(OpenSCADObject):
    def __init__(self, poly1=None, poly2=None, eps=None, st=None, **kwargs):
       super().__init__("_are_polygons_equal", {"poly1" : poly1, "poly2" : poly2, "eps" : eps, "st" : st, **kwargs})

class _is_polygon_in_list(OpenSCADObject):
    def __init__(self, poly=None, polys=None, **kwargs):
       super().__init__("_is_polygon_in_list", {"poly" : poly, "polys" : polys, **kwargs})

class ___is_polygon_in_list(OpenSCADObject):
    def __init__(self, poly=None, polys=None, i=None, **kwargs):
       super().__init__("___is_polygon_in_list", {"poly" : poly, "polys" : polys, "i" : i, **kwargs})

class hull(OpenSCADObject):
    def __init__(self, points=None, **kwargs):
       super().__init__("hull", {"points" : points, **kwargs})

class _backtracking(OpenSCADObject):
    def __init__(self, i=None, points=None, h=None, t=None, m=None, all=None, **kwargs):
       super().__init__("_backtracking", {"i" : i, "points" : points, "h" : h, "t" : t, "m" : m, "all" : all, **kwargs})

class _is_cw(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, all=None, **kwargs):
       super().__init__("_is_cw", {"a" : a, "b" : b, "c" : c, "all" : all, **kwargs})

class hull2d_path(OpenSCADObject):
    def __init__(self, points=None, all=None, **kwargs):
       super().__init__("hull2d_path", {"points" : points, "all" : all, **kwargs})

class _hull_collinear(OpenSCADObject):
    def __init__(self, points=None, **kwargs):
       super().__init__("_hull_collinear", {"points" : points, **kwargs})

class hull3d_faces(OpenSCADObject):
    def __init__(self, points=None, **kwargs):
       super().__init__("hull3d_faces", {"points" : points, **kwargs})

class _hull3d_iterative(OpenSCADObject):
    def __init__(self, points=None, triangles=None, planes=None, remaining=None, _i=None, **kwargs):
       super().__init__("_hull3d_iterative", {"points" : points, "triangles" : triangles, "planes" : planes, "remaining" : remaining, "_i" : _i, **kwargs})

class _remove_internal_edges(OpenSCADObject):
    def __init__(self, halfedges=None, **kwargs):
       super().__init__("_remove_internal_edges", {"halfedges" : halfedges, **kwargs})

class _find_first_noncoplanar(OpenSCADObject):
    def __init__(self, plane=None, points=None, i=None, **kwargs):
       super().__init__("_find_first_noncoplanar", {"plane" : plane, "points" : points, "i" : i, **kwargs})

class is_polygon_convex(OpenSCADObject):
    def __init__(self, poly=None, eps=None, **kwargs):
       super().__init__("is_polygon_convex", {"poly" : poly, "eps" : eps, **kwargs})

class convex_distance(OpenSCADObject):
    def __init__(self, points1=None, points2=None, eps=None, **kwargs):
       super().__init__("convex_distance", {"points1" : points1, "points2" : points2, "eps" : eps, **kwargs})

class _GJK_distance(OpenSCADObject):
    def __init__(self, points1=None, points2=None, eps=None, lbd=None, d=None, simplex=None, **kwargs):
       super().__init__("_GJK_distance", {"points1" : points1, "points2" : points2, "eps" : eps, "lbd" : lbd, "d" : d, "simplex" : simplex, **kwargs})

class convex_collision(OpenSCADObject):
    def __init__(self, points1=None, points2=None, eps=None, **kwargs):
       super().__init__("convex_collision", {"points1" : points1, "points2" : points2, "eps" : eps, **kwargs})

class _GJK_collide(OpenSCADObject):
    def __init__(self, points1=None, points2=None, d=None, simplex=None, eps=None, **kwargs):
       super().__init__("_GJK_collide", {"points1" : points1, "points2" : points2, "d" : d, "simplex" : simplex, "eps" : eps, **kwargs})

class _closest_simplex(OpenSCADObject):
    def __init__(self, s=None, eps=None, **kwargs):
       super().__init__("_closest_simplex", {"s" : s, "eps" : eps, **kwargs})

class _closest_s1(OpenSCADObject):
    def __init__(self, s=None, eps=None, **kwargs):
       super().__init__("_closest_s1", {"s" : s, "eps" : eps, **kwargs})

class _closest_s2(OpenSCADObject):
    def __init__(self, s=None, eps=None, **kwargs):
       super().__init__("_closest_s2", {"s" : s, "eps" : eps, **kwargs})

class _closest_s3(OpenSCADObject):
    def __init__(self, s=None, eps=None, **kwargs):
       super().__init__("_closest_s3", {"s" : s, "eps" : eps, **kwargs})

class _tri_normal(OpenSCADObject):
    def __init__(self, tri=None, **kwargs):
       super().__init__("_tri_normal", {"tri" : tri, **kwargs})

class _support_diff(OpenSCADObject):
    def __init__(self, p1=None, p2=None, d=None, **kwargs):
       super().__init__("_support_diff", {"p1" : p1, "p2" : p2, "d" : d, **kwargs})

class rot_decode(OpenSCADObject):
    def __init__(self, M=None, long=None, **kwargs):
       super().__init__("rot_decode", {"M" : M, "long" : long, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/joiners.scad'}", use_not_include=False)

class half_joiner_clear(OpenSCADObject):
    def __init__(self, l=None, w=None, ang=None, clearance=None, overlap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner_clear", {"l" : l, "w" : w, "ang" : ang, "clearance" : clearance, "overlap" : overlap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class half_joiner(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class half_joiner2(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner2", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class joiner_clear(OpenSCADObject):
    def __init__(self, l=None, w=None, ang=None, clearance=None, overlap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("joiner_clear", {"l" : l, "w" : w, "ang" : ang, "clearance" : clearance, "overlap" : overlap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class joiner(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("joiner", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class dovetail(OpenSCADObject):
    def __init__(self, gender=None, width=None, height=None, slide=None, h=None, w=None, angle=None, slope=None, taper=None, back_width=None, chamfer=None, extra=None, r=None, radius=None, round=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("dovetail", {"gender" : gender, "width" : width, "height" : height, "slide" : slide, "h" : h, "w" : w, "angle" : angle, "slope" : slope, "taper" : taper, "back_width" : back_width, "chamfer" : chamfer, "extra" : extra, "r" : r, "radius" : radius, "round" : round, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _pin_nub(OpenSCADObject):
    def __init__(self, r=None, nub=None, h=None, **kwargs):
       super().__init__("_pin_nub", {"r" : r, "nub" : nub, "h" : h, **kwargs})

class _pin_slot(OpenSCADObject):
    def __init__(self, l=None, r=None, t=None, d=None, nub=None, depth=None, stretch=None, **kwargs):
       super().__init__("_pin_slot", {"l" : l, "r" : r, "t" : t, "d" : d, "nub" : nub, "depth" : depth, "stretch" : stretch, **kwargs})

class _pin_shaft(OpenSCADObject):
    def __init__(self, r=None, lStraight=None, nub=None, nubscale=None, stretch=None, d=None, pointed=None, **kwargs):
       super().__init__("_pin_shaft", {"r" : r, "lStraight" : lStraight, "nub" : nub, "nubscale" : nubscale, "stretch" : stretch, "d" : d, "pointed" : pointed, **kwargs})

class snap_pin(OpenSCADObject):
    def __init__(self, size=None, r=None, radius=None, d=None, diameter=None, l=None, length=None, nub_depth=None, snap=None, thickness=None, clearance=None, preload=None, pointed=None, anchor=None, spin=None, orient=None, center=None, **kwargs):
       super().__init__("snap_pin", {"size" : size, "r" : r, "radius" : radius, "d" : d, "diameter" : diameter, "l" : l, "length" : length, "nub_depth" : nub_depth, "snap" : snap, "thickness" : thickness, "clearance" : clearance, "preload" : preload, "pointed" : pointed, "anchor" : anchor, "spin" : spin, "orient" : orient, "center" : center, **kwargs})

class snap_pin_socket(OpenSCADObject):
    def __init__(self, size=None, r=None, radius=None, l=None, length=None, d=None, diameter=None, nub_depth=None, snap=None, fixed=None, pointed=None, fins=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("snap_pin_socket", {"size" : size, "r" : r, "radius" : radius, "l" : l, "length" : length, "d" : d, "diameter" : diameter, "nub_depth" : nub_depth, "snap" : snap, "fixed" : fixed, "pointed" : pointed, "fins" : fins, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rabbit_clip(OpenSCADObject):
    def __init__(self, type=None, length=None, width=None, snap=None, thickness=None, depth=None, compression=None, clearance=None, lock=None, lock_clearance=None, splinesteps=None, anchor=None, orient=None, spin=None, **kwargs):
       super().__init__("rabbit_clip", {"type" : type, "length" : length, "width" : width, "snap" : snap, "thickness" : thickness, "depth" : depth, "compression" : compression, "clearance" : clearance, "lock" : lock, "lock_clearance" : lock_clearance, "splinesteps" : splinesteps, "anchor" : anchor, "orient" : orient, "spin" : spin, **kwargs})

class half_joiner_clear(OpenSCADObject):
    def __init__(self, l=None, w=None, ang=None, clearance=None, overlap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner_clear", {"l" : l, "w" : w, "ang" : ang, "clearance" : clearance, "overlap" : overlap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class half_joiner(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class half_joiner2(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("half_joiner2", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class joiner_clear(OpenSCADObject):
    def __init__(self, l=None, w=None, ang=None, clearance=None, overlap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("joiner_clear", {"l" : l, "w" : w, "ang" : ang, "clearance" : clearance, "overlap" : overlap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class joiner(OpenSCADObject):
    def __init__(self, l=None, w=None, base=None, ang=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("joiner", {"l" : l, "w" : w, "base" : base, "ang" : ang, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class dovetail(OpenSCADObject):
    def __init__(self, gender=None, width=None, height=None, slide=None, h=None, w=None, angle=None, slope=None, taper=None, back_width=None, chamfer=None, extra=None, r=None, radius=None, round=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("dovetail", {"gender" : gender, "width" : width, "height" : height, "slide" : slide, "h" : h, "w" : w, "angle" : angle, "slope" : slope, "taper" : taper, "back_width" : back_width, "chamfer" : chamfer, "extra" : extra, "r" : r, "radius" : radius, "round" : round, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _pin_size(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("_pin_size", {"size" : size, **kwargs})

class snap_pin(OpenSCADObject):
    def __init__(self, size=None, r=None, radius=None, d=None, diameter=None, l=None, length=None, nub_depth=None, snap=None, thickness=None, clearance=None, preload=None, pointed=None, anchor=None, spin=None, orient=None, center=None, **kwargs):
       super().__init__("snap_pin", {"size" : size, "r" : r, "radius" : radius, "d" : d, "diameter" : diameter, "l" : l, "length" : length, "nub_depth" : nub_depth, "snap" : snap, "thickness" : thickness, "clearance" : clearance, "preload" : preload, "pointed" : pointed, "anchor" : anchor, "spin" : spin, "orient" : orient, "center" : center, **kwargs})

class snap_pin_socket(OpenSCADObject):
    def __init__(self, size=None, r=None, radius=None, l=None, length=None, d=None, diameter=None, nub_depth=None, snap=None, fixed=None, pointed=None, fins=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("snap_pin_socket", {"size" : size, "r" : r, "radius" : radius, "l" : l, "length" : length, "d" : d, "diameter" : diameter, "nub_depth" : nub_depth, "snap" : snap, "fixed" : fixed, "pointed" : pointed, "fins" : fins, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rabbit_clip(OpenSCADObject):
    def __init__(self, type=None, length=None, width=None, snap=None, thickness=None, depth=None, compression=None, clearance=None, lock=None, lock_clearance=None, splinesteps=None, anchor=None, orient=None, spin=None, **kwargs):
       super().__init__("rabbit_clip", {"type" : type, "length" : length, "width" : width, "snap" : snap, "thickness" : thickness, "depth" : depth, "compression" : compression, "clearance" : clearance, "lock" : lock, "lock_clearance" : lock_clearance, "splinesteps" : splinesteps, "anchor" : anchor, "orient" : orient, "spin" : spin, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/vnf.scad'}", use_not_include=False)

EMPTY_VNF = OpenSCADConstant('EMPTY_VNF')
_vnf_validate_errs = OpenSCADConstant('_vnf_validate_errs')
class vnf_polyhedron(OpenSCADObject):
    def __init__(self, vnf=None, convexity=None, extent=None, cp=None, anchor=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("vnf_polyhedron", {"vnf" : vnf, "convexity" : convexity, "extent" : extent, "cp" : cp, "anchor" : anchor, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class vnf_wireframe(OpenSCADObject):
    def __init__(self, vnf=None, width=None, **kwargs):
       super().__init__("vnf_wireframe", {"vnf" : vnf, "width" : width, **kwargs})

class _show_vertices(OpenSCADObject):
    def __init__(self, vertices=None, size=None, **kwargs):
       super().__init__("_show_vertices", {"vertices" : vertices, "size" : size, **kwargs})

class _show_faces(OpenSCADObject):
    def __init__(self, vertices=None, faces=None, size=None, **kwargs):
       super().__init__("_show_faces", {"vertices" : vertices, "faces" : faces, "size" : size, **kwargs})

class debug_vnf(OpenSCADObject):
    def __init__(self, vnf=None, faces=None, vertices=None, opacity=None, size=None, convexity=None, **kwargs):
       super().__init__("debug_vnf", {"vnf" : vnf, "faces" : faces, "vertices" : vertices, "opacity" : opacity, "size" : size, "convexity" : convexity, **kwargs})

class vnf_validate(OpenSCADObject):
    def __init__(self, vnf=None, size=None, show_warns=None, check_isects=None, **kwargs):
       super().__init__("vnf_validate", {"vnf" : vnf, "size" : size, "show_warns" : show_warns, "check_isects" : check_isects, **kwargs})

class vnf_vertex_array(OpenSCADObject):
    def __init__(self, points=None, caps=None, cap1=None, cap2=None, col_wrap=None, row_wrap=None, reverse=None, style=None, **kwargs):
       super().__init__("vnf_vertex_array", {"points" : points, "caps" : caps, "cap1" : cap1, "cap2" : cap2, "col_wrap" : col_wrap, "row_wrap" : row_wrap, "reverse" : reverse, "style" : style, **kwargs})

class vnf_tri_array(OpenSCADObject):
    def __init__(self, points=None, row_wrap=None, reverse=None, **kwargs):
       super().__init__("vnf_tri_array", {"points" : points, "row_wrap" : row_wrap, "reverse" : reverse, **kwargs})

class vnf_join(OpenSCADObject):
    def __init__(self, vnfs=None, **kwargs):
       super().__init__("vnf_join", {"vnfs" : vnfs, **kwargs})

class vnf_from_polygons(OpenSCADObject):
    def __init__(self, polygons=None, **kwargs):
       super().__init__("vnf_from_polygons", {"polygons" : polygons, **kwargs})

class _path_path_closest_vertices(OpenSCADObject):
    def __init__(self, path1=None, path2=None, **kwargs):
       super().__init__("_path_path_closest_vertices", {"path1" : path1, "path2" : path2, **kwargs})

class _join_paths_at_vertices(OpenSCADObject):
    def __init__(self, path1=None, path2=None, v1=None, v2=None, **kwargs):
       super().__init__("_join_paths_at_vertices", {"path1" : path1, "path2" : path2, "v1" : v1, "v2" : v2, **kwargs})

class _cleave_connected_region(OpenSCADObject):
    def __init__(self, region=None, eps=None, **kwargs):
       super().__init__("_cleave_connected_region", {"region" : region, "eps" : eps, **kwargs})

class _polyHoles(OpenSCADObject):
    def __init__(self, outer=None, holes=None, extremes=None, eps=None, n=None, **kwargs):
       super().__init__("_polyHoles", {"outer" : outer, "holes" : holes, "extremes" : extremes, "eps" : eps, "n" : n, **kwargs})

class _bridge(OpenSCADObject):
    def __init__(self, pt=None, outer=None, eps=None, **kwargs):
       super().__init__("_bridge", {"pt" : pt, "outer" : outer, "eps" : eps, **kwargs})

class vnf_from_region(OpenSCADObject):
    def __init__(self, region=None, transform=None, reverse=None, **kwargs):
       super().__init__("vnf_from_region", {"region" : region, "transform" : transform, "reverse" : reverse, **kwargs})

class is_vnf(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_vnf", {"x" : x, **kwargs})

class is_vnf_list(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_vnf_list", {"x" : x, **kwargs})

class vnf_vertices(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_vertices", {"vnf" : vnf, **kwargs})

class vnf_faces(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_faces", {"vnf" : vnf, **kwargs})

class vnf_reverse_faces(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_reverse_faces", {"vnf" : vnf, **kwargs})

class vnf_quantize(OpenSCADObject):
    def __init__(self, vnf=None, q=None, **kwargs):
       super().__init__("vnf_quantize", {"vnf" : vnf, "q" : q, **kwargs})

class vnf_merge_points(OpenSCADObject):
    def __init__(self, vnf=None, eps=None, **kwargs):
       super().__init__("vnf_merge_points", {"vnf" : vnf, "eps" : eps, **kwargs})

class vnf_drop_unused_points(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_drop_unused_points", {"vnf" : vnf, **kwargs})

class _link_indicator(OpenSCADObject):
    def __init__(self, l=None, imin=None, imax=None, **kwargs):
       super().__init__("_link_indicator", {"l" : l, "imin" : imin, "imax" : imax, **kwargs})

class vnf_triangulate(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_triangulate", {"vnf" : vnf, **kwargs})

class vnf_slice(OpenSCADObject):
    def __init__(self, vnf=None, dir=None, cuts=None, **kwargs):
       super().__init__("vnf_slice", {"vnf" : vnf, "dir" : dir, "cuts" : cuts, **kwargs})

class _split_polygon_at_x(OpenSCADObject):
    def __init__(self, poly=None, x=None, **kwargs):
       super().__init__("_split_polygon_at_x", {"poly" : poly, "x" : x, **kwargs})

class _split_2dpolygons_at_each_x(OpenSCADObject):
    def __init__(self, polys=None, xs=None, _i=None, **kwargs):
       super().__init__("_split_2dpolygons_at_each_x", {"polys" : polys, "xs" : xs, "_i" : _i, **kwargs})

class _slice_3dpolygons(OpenSCADObject):
    def __init__(self, polys=None, dir=None, cuts=None, **kwargs):
       super().__init__("_slice_3dpolygons", {"polys" : polys, "dir" : dir, "cuts" : cuts, **kwargs})

class vnf_volume(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_volume", {"vnf" : vnf, **kwargs})

class vnf_area(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("vnf_area", {"vnf" : vnf, **kwargs})

class _vnf_centroid(OpenSCADObject):
    def __init__(self, vnf=None, eps=None, **kwargs):
       super().__init__("_vnf_centroid", {"vnf" : vnf, "eps" : eps, **kwargs})

class vnf_halfspace(OpenSCADObject):
    def __init__(self, plane=None, vnf=None, closed=None, **kwargs):
       super().__init__("vnf_halfspace", {"plane" : plane, "vnf" : vnf, "closed" : closed, **kwargs})

class _assemble_paths(OpenSCADObject):
    def __init__(self, vertices=None, edges=None, paths=None, i=None, **kwargs):
       super().__init__("_assemble_paths", {"vertices" : vertices, "edges" : edges, "paths" : paths, "i" : i, **kwargs})

class _vnfcut(OpenSCADObject):
    def __init__(self, plane=None, vertices=None, vertexmap=None, inside=None, faces=None, vertcount=None, newfaces=None, newedges=None, newvertices=None, i=None, **kwargs):
       super().__init__("_vnfcut", {"plane" : plane, "vertices" : vertices, "vertexmap" : vertexmap, "inside" : inside, "faces" : faces, "vertcount" : vertcount, "newfaces" : newfaces, "newedges" : newedges, "newvertices" : newvertices, "i" : i, **kwargs})

class _triangulate_planar_convex_polygons(OpenSCADObject):
    def __init__(self, polys=None, **kwargs):
       super().__init__("_triangulate_planar_convex_polygons", {"polys" : polys, **kwargs})

class vnf_bend(OpenSCADObject):
    def __init__(self, vnf=None, r=None, d=None, axis=None, **kwargs):
       super().__init__("vnf_bend", {"vnf" : vnf, "r" : r, "d" : d, "axis" : axis, **kwargs})

class vnf_validate(OpenSCADObject):
    def __init__(self, vnf=None, show_warns=None, check_isects=None, **kwargs):
       super().__init__("vnf_validate", {"vnf" : vnf, "show_warns" : show_warns, "check_isects" : check_isects, **kwargs})

class _vnf_validate_err(OpenSCADObject):
    def __init__(self, name=None, extra=None, **kwargs):
       super().__init__("_vnf_validate_err", {"name" : name, "extra" : extra, **kwargs})

class _pts_not_reported(OpenSCADObject):
    def __init__(self, pts=None, varr=None, reports=None, **kwargs):
       super().__init__("_pts_not_reported", {"pts" : pts, "varr" : varr, "reports" : reports, **kwargs})

class _edge_not_reported(OpenSCADObject):
    def __init__(self, edge=None, varr=None, reports=None, **kwargs):
       super().__init__("_edge_not_reported", {"edge" : edge, "varr" : varr, "reports" : reports, **kwargs})




#! /usr/bin/env python

from solid2 import *

set_global_fn(72)

difference()(
    cube([10, 20, 30]),
    sphere(10)
).save_as_scad()

# this is very very basic but you can make this as complex as in OpenSCAD.
# You can really just use "python as OpenSCAD". You just need to call
# .save_as_scad() at last to write it to a *.scad file.
#
# If you execute this file it will create examples/01-basics.scad with the this
# content:
#
#        $fn = 72;
#
#        difference() {
#            cube(size = [10, 20, 30]);
#            sphere(r = 10);
#        }
#



#! /usr/bin/env python

from pathlib import Path

from solid2.core.utils import escape_openscad_identifier as escape
from solid2.libs.py_scadparser import scad_parser

headerTemplate = """\
from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{{Path(__file__).parent.parent / '{scadFile}'}}", use_not_include={use_not_include})

"""

constantTemplate = "{name} = OpenSCADConstant('{name}')"

callableTemplate = """\
class {name}(OpenSCADObject):
    def __init__({paramStr}):
       super().__init__({initStr})

"""

def generateStub(scadFile, outputDir, use_not_include):
    def generateHeader():
        return headerTemplate.format(__file__=__file__,
                                     scadFile=scadFile,
                                     use_not_include=use_not_include)

    def generateConstant(c):
        return constantTemplate.format(name=escape(c.name)) + "\n"

    def generateCallable(c):
        name = escape(c.name)
        paramNames = [escape(p.name) for p in c.parameters]

        paramStr = ", ".join(["self"] +
                             [f"{p}=None" for p in paramNames] +
                             ["**kwargs"])
        initList = [f'"{p}" : {p}' for p in paramNames]
        initList.append("**kwargs")
        initStr = f'"{name}", {{{", ".join(initList)}}}'
        return callableTemplate.format(name=name, paramStr=paramStr, initStr=initStr)

    modules, functions, global_vars = scad_parser.parseFile(scadFile)

    escaped_filename = escape(scadFile.stem) + ".py"
    with open(outputDir / escaped_filename, "w") as f:
        f.write(generateHeader())

        for c in global_vars:
            f.write(generateConstant(c))

        for c in modules + functions:
            f.write(generateCallable(c))

def generateInit(inDir, outputDir, packageName):
    import os
    try:
        os.mkdir(outputDir / packageName)
    except FileExistsError:
        pass

    stubFile = outputDir / packageName / "__init__.py"

    with open(stubFile, "w") as std_f:
        for f in inDir.iterdir():
            if not f.suffix == ".scad":
                continue

            std_f.write(f"from . import {escape(Path(f.name).stem)}\n")


if __name__ == "__main__":
    from argparse import ArgumentParser
    parser = ArgumentParser(description="generates a solidpython2 extensions "
                                        "from a OpenSCAD library")
    parser.add_argument("scadInDir",
                        help="directory containing the scad (library) files")
    parser.add_argument("packageName",
                        help="name of the generated python package")
    parser.add_argument("-i", "--include", action='store_false', default=True,
                        help="use OpenSCADs 'include' to import the library. "
                             "Otherwiese OpenSCADs 'use' is used")

    args = parser.parse_args()

    generateInit(Path(args.scadInDir), Path("."), args.packageName)

    for f in Path(args.scadInDir).iterdir():
        if not f.suffix == ".scad":
            continue

        generateStub(f, Path(".") / args.packageName, args.include)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/linear_bearings.scad'}", use_not_include=False)

class linear_bearing_housing(OpenSCADObject):
    def __init__(self, d=None, l=None, tab=None, gap=None, wall=None, tabwall=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("linear_bearing_housing", {"d" : d, "l" : l, "tab" : tab, "gap" : gap, "wall" : wall, "tabwall" : tabwall, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class lmXuu_housing(OpenSCADObject):
    def __init__(self, size=None, tab=None, gap=None, wall=None, tabwall=None, screwsize=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("lmXuu_housing", {"size" : size, "tab" : tab, "gap" : gap, "wall" : wall, "tabwall" : tabwall, "screwsize" : screwsize, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class get_lmXuu_bearing_diam(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_lmXuu_bearing_diam", {"size" : size, **kwargs})

class get_lmXuu_bearing_length(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_lmXuu_bearing_length", {"size" : size, **kwargs})




from .accessSyntaxBase import AccessSyntaxBase
from .operatorBase import OperatorBase

#don't do relative imports on the global scope to be able to import this file
#from "everywhere"

class ObjectBaseInterface(AccessSyntaxBase, OperatorBase):
    def _render(self):
        return ""

    def __repr__(self):
        return self.as_scad()

    def as_scad(self):
        from .scad_render import scad_render
        return scad_render(self)[:-1]

    def save_as_scad(self, filename='', outdir=''):
        from .scad_render import scad_render_to_file
        return scad_render_to_file(self, filename, outdir)

    def save_as_stl(self, filename='', outdir=''):
        from .scad_render import render_to_stl_file
        return render_to_stl_file(self, filename)

class ObjectBase(ObjectBaseInterface):
    def __init__(self):
        self.children = []

    def add(self, c):
        def _add(c):
            assert(hasattr(c, "_render"))
            self.children += [c]

        if isinstance(c, list):
            for cc in c:
                _add(cc)
        else:
            _add(c)

        return self

    def _render(self):
        s = ''
        for c in self.children:
            s += c._render()
        return s

    def __call__(self, *args):
        #translate(...)(cube())
        #this adds cube() to translate.children
        for a in args:
            self.add(a)
        return self

class OpenSCADObject(ObjectBase):
    def __init__(self, name, params):
        super().__init__()
        self.name = name
        self.params = params

    def _render(self):
        """
            returns the scad code for a given node tuple consiting of name,
            params and children list.

            -> translate(v = [1, 2, 3]) {children[0]; children[1]; ...};\n
        """
        from .utils import indent
        s = self.generate_scad_head()

        if self.children:
            s += " {\n"
            for child in self.children:
                s += indent(child._render())
            s += "}"
        else:
            s += ";"

        return s + "\n"

    def generate_scad_head(self):
        """
            for a given function name and dict of params it returns:
                {name}(p1=v1, p2=v2,...)
                -> translate(v = [1, 2, 3])
        """
        from .utils import unescape_openscad_identifier, py2openscad
        from ..config import config

        param_strings = []
        for p in sorted(self.params.keys()):
            if self.params[p] is None:
                continue

            if config.use_implicit_builtins and \
                        isinstance(self.params[p], OpenSCADParameterFunction):

                param_strings.append(self.params[p]._render())
            else:
                scad_value = py2openscad(self.params[p])
                scad_identifier = unescape_openscad_identifier(p)

                param_strings.append(f'{scad_identifier} = {scad_value}')

        scad_identifier = unescape_openscad_identifier(self.name)
        param_str = ", ".join(param_strings)

        return f'{scad_identifier}({param_str})'

class OpenSCADConstant:
    def __init__(self, value):
        self.value = value

        from .utils import escape_openscad_identifier
        self.__doc__ = escape_openscad_identifier(value)

    def __repr__(self):
        return f'{self.value}'

    def __operator_base__(self, op, other):
        return OpenSCADConstant(f'({self} {op} {other})')

    def __roperator_base__(self, op, other):
        return OpenSCADConstant(f'({other} {op} {self})')

    def __unary_operator_base__(self, op):
        return OpenSCADConstant(f'({op}{self})')

    def __illegal_operator__(self):
       raise Exception("You can't compare a OpenSCADConstant with something else, " +\
                       "because we don't know the customized value at " +\
                       "SolidPythons runtime because it might get customized " +\
                       "at OpenSCAD runtime.")

    #basic operators +, -, *, /, %, **
    def __add__(self, other): return self.__operator_base__("+", other)
    def __sub__(self, other): return self.__operator_base__("-", other)
    def __mul__(self, other): return self.__operator_base__("*", other)
    def __mod__(self, other): return self.__operator_base__("%", other)
    def __pow__(self, other): return self.__operator_base__("^", other)
    def __truediv__(self, other): return self.__operator_base__("/", other)

    def __radd__(self, other): return self.__roperator_base__("+", other)
    def __rsub__(self, other): return self.__roperator_base__("-", other)
    def __rmul__(self, other): return self.__roperator_base__("*", other)
    def __rmod__(self, other): return self.__roperator_base__("%", other)
    def __rpow__(self, other): return self.__roperator_base__("^", other)
    def __rtruediv__(self, other): return self.__roperator_base__("/", other)

    #unary operators
    def __neg__(self): return self.__unary_operator_base__("-")

    #other operators
    def __abs__(self): return OpenSCADConstant(f'abs({self})')

    #"illegal" operators
    def __eq__(self, _): return self.__illegal_operator__()
    def __ne__(self, _): return self.__illegal_operator__()
    def __le__(self, _): return self.__illegal_operator__()
    def __ge__(self, _): return self.__illegal_operator__()
    def __lt__(self, _): return self.__illegal_operator__()
    def __gt__(self, _): return self.__illegal_operator__()

    #do not allow to evaluate to bool
    def __bool__(self):
        raise Exception("You can't use scad variables as truth statement because " +\
                        "we don't know the value of a customized variable at " +\
                        "SolidPython runtime.")

    def _render(self):
        from textwrap import dedent
        return dedent(self.value)

import math
sin = lambda x: OpenSCADConstant(f'sin({x})') if isinstance(x, OpenSCADConstant) else math.sin(x)
cos = lambda x: OpenSCADConstant(f'cos({x})') if isinstance(x, OpenSCADConstant) else math.cos(x)
tan = lambda x: OpenSCADConstant(f'tan({x})') if isinstance(x, OpenSCADConstant) else math.tan(x)
asin = lambda x: OpenSCADConstant(f'asin({x})') if isinstance(x, OpenSCADConstant) else math.asin(x)
acos = lambda x: OpenSCADConstant(f'acos({x})') if isinstance(x, OpenSCADConstant) else math.acos(x)
atan = lambda x: OpenSCADConstant(f'atan({x})') if isinstance(x, OpenSCADConstant) else math.atan(x)
sqrt = lambda x: OpenSCADConstant(f'sqrt({x})') if isinstance(x, OpenSCADConstant) else math.sqrt(x)
not_ = lambda x: OpenSCADConstant(f'!{x}')

def scad_inline(code):
    return OpenSCADConstant(code)

class OpenSCADParameterFunction(OpenSCADConstant):
    pass

def scad_inline_parameter_func(code):
    return OpenSCADParameterFunction(code)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/polyhedra.scad'}", use_not_include=False)

_tribonacci = OpenSCADConstant('_tribonacci')
_polyhedra_ = OpenSCADConstant('_polyhedra_')
_stellated_polyhedra_ = OpenSCADConstant('_stellated_polyhedra_')
class regular_polyhedron(OpenSCADObject):
    def __init__(self, name=None, index=None, type=None, faces=None, facetype=None, hasfaces=None, side=None, ir=None, mr=None, _or=None, r=None, d=None, anchor=None, rounding=None, repeat=None, facedown=None, draw=None, rotate_children=None, stellate=None, longside=None, h=None, **kwargs):
       super().__init__("regular_polyhedron", {"name" : name, "index" : index, "type" : type, "faces" : faces, "facetype" : facetype, "hasfaces" : hasfaces, "side" : side, "ir" : ir, "mr" : mr, "_or" : _or, "r" : r, "d" : d, "anchor" : anchor, "rounding" : rounding, "repeat" : repeat, "facedown" : facedown, "draw" : draw, "rotate_children" : rotate_children, "stellate" : stellate, "longside" : longside, "h" : h, **kwargs})

class _unique_groups(OpenSCADObject):
    def __init__(self, m=None, **kwargs):
       super().__init__("_unique_groups", {"m" : m, **kwargs})

class _even_perms(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_even_perms", {"v" : v, **kwargs})

class _all_perms(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_all_perms", {"v" : v, **kwargs})

class _point_ref(OpenSCADObject):
    def __init__(self, points=None, sign=None, **kwargs):
       super().__init__("_point_ref", {"points" : points, "sign" : sign, **kwargs})

class regular_polyhedron_info(OpenSCADObject):
    def __init__(self, info=None, name=None, index=None, type=None, faces=None, facetype=None, hasfaces=None, side=None, ir=None, mr=None, _or=None, r=None, d=None, anchor=None, facedown=None, stellate=None, longside=None, h=None, **kwargs):
       super().__init__("regular_polyhedron_info", {"info" : info, "name" : name, "index" : index, "type" : type, "faces" : faces, "facetype" : facetype, "hasfaces" : hasfaces, "side" : side, "ir" : ir, "mr" : mr, "_or" : _or, "r" : r, "d" : d, "anchor" : anchor, "facedown" : facedown, "stellate" : stellate, "longside" : longside, "h" : h, **kwargs})

class _stellate_faces(OpenSCADObject):
    def __init__(self, scalefactor=None, stellate=None, vertices=None, faces_normals=None, **kwargs):
       super().__init__("_stellate_faces", {"scalefactor" : scalefactor, "stellate" : stellate, "vertices" : vertices, "faces_normals" : faces_normals, **kwargs})

class _trapezohedron(OpenSCADObject):
    def __init__(self, faces=None, r=None, side=None, longside=None, h=None, d=None, **kwargs):
       super().__init__("_trapezohedron", {"faces" : faces, "r" : r, "side" : side, "longside" : longside, "h" : h, "d" : d, **kwargs})

class _facenormal(OpenSCADObject):
    def __init__(self, pts=None, face=None, **kwargs):
       super().__init__("_facenormal", {"pts" : pts, "face" : face, **kwargs})

class _full_faces(OpenSCADObject):
    def __init__(self, pts=None, faces=None, **kwargs):
       super().__init__("_full_faces", {"pts" : pts, "faces" : faces, **kwargs})

